@@ -556,9 +556,9 @@ QString qtractorClip::toolTip (void) const
 	if (pSession) {
 		sToolTip += '\n';
 		qtractorTimeScale *pTimeScale = pSession->timeScale();
-		sToolTip += QObject::tr("Start / End:\t%1 / %2\nOffs. / Len.:\t%3 / %4")
+		sToolTip += QObject::tr("Start / End:\t%1 / %2\nOffset / Length:\t%3 / %4")
 			.arg(pTimeScale->textFromFrame(m_iClipStart))
-			.arg(pTimeScale->textFromFrame(m_iClipStart + m_iClipLength))
+			.arg(pTimeScale->textFromFrame(m_iClipStart  + m_iClipLength))
 			.arg(pTimeScale->textFromFrame(m_iClipStart, true, m_iClipOffset))
 			.arg(pTimeScale->textFromFrame(m_iClipStart, true, m_iClipLength));
 	}
@@ -1026,4 +1026,4 @@ qtractorClip::TakeInfo *qtractorClip::takeInfo (void) const
 }
 
 
-// end of qtractorClip.h
+// end of qtractorClip.cpp
