@@ -1,7 +1,7 @@
 // qtractorDocument.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2012, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -81,6 +81,9 @@ QString qtractorDocument::g_sArchiveExt  = "qtz";
 // Extracted archive paths (static).
 QStringList qtractorDocument::g_extractedArchives;
 
+// Extra-ordinary archive files (static).
+qtractorDocument *qtractorDocument::g_pArchive = NULL;
+
 
 // Constructor.
 qtractorDocument::qtractorDocument ( QDomDocument *pDocument,
@@ -167,6 +170,7 @@ bool qtractorDocument::load ( const QString& sFilename, Flags flags )
 	if (m_pZipFile) {
 		delete m_pZipFile;
 		m_pZipFile = NULL;
+		g_pArchive = NULL;
 	}
 #endif
 
@@ -242,6 +246,7 @@ bool qtractorDocument::save ( const QString& sFilename, Flags flags )
 	if (m_pZipFile) {
 		delete m_pZipFile;
 		m_pZipFile = NULL;
+		g_pArchive = NULL;
 	}
 #endif
 
@@ -260,6 +265,7 @@ bool qtractorDocument::save ( const QString& sFilename, Flags flags )
 			return false;
 		}
 		sDocname = m_sName + '.' + g_sDefaultExt;
+		g_pArchive = this;
 	}
 #endif
 
@@ -283,11 +289,13 @@ bool qtractorDocument::save ( const QString& sFilename, Flags flags )
 #ifdef CONFIG_LIBZ
 	// Commit to archive.
 	if (m_pZipFile) {
+		// The session document itself, at last...
 		m_pZipFile->addFile(sDocname, m_sName + '/' + sDocname);
 		m_pZipFile->processAll();
 		m_pZipFile->close();
 		delete m_pZipFile;
 		m_pZipFile = NULL;
+		g_pArchive = NULL;
 		// Kill temporary, if didn't exist...
 		if (bRemove) file.remove();
 	}
@@ -384,4 +392,15 @@ void qtractorDocument::clearExtractedArchives ( bool bRemove )
 }
 
 
+//-------------------------------------------------------------------------
+// qtractorDocument -- extra-ordinary archive files management.
+//
+
+QString qtractorDocument::addArchiveFile ( const QString& sFilename )
+{
+	return (g_pArchive ? g_pArchive->addFile(sFilename) : sFilename);
+}
+
+
+
 // end of qtractorDocument.cpp
