@@ -543,12 +543,16 @@ void qtractorFileListView::openFile (void)
 void qtractorFileListView::newGroup (void)
 {
 	qtractorFileGroupItem *pParentItem = currentGroupItem();
+	if (!pParentItem->isOpen())
+		pParentItem = pParentItem->groupItem();
 	qtractorFileGroupItem *pGroupItem
 		= addGroupItem(tr("New Group"), pParentItem);
-	if (pParentItem)
-		pParentItem->setOpen(true);
-	if (pGroupItem)
+	if (pGroupItem) {
+		pParentItem = pGroupItem->groupItem();
+		if (pParentItem)
+			pParentItem->setOpen(true);
 		editItem(pGroupItem, 0);
+	}
 }
 
 
@@ -630,7 +634,7 @@ void qtractorFileListView::renameItem (void)
 
 
 // Remove current group/file item.
-void qtractorFileListView::deleteItem (void)
+void qtractorFileListView::removeItem (void)
 {
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
@@ -642,12 +646,12 @@ void qtractorFileListView::deleteItem (void)
 
 	int iUpdate = 0;
 	QList<QTreeWidgetItem *> items = selectedItems();
-	if (items.count() > 1) {
+	if (items.count() > 0) {
 		// Prompt user if he/she's sure about this...
 		if (pOptions->bConfirmRemove) {
 			if (QMessageBox::warning(this,
 				tr("Warning") + " - " QTRACTOR_TITLE,
-				tr("About to remove %1 file items.\n\n"
+				tr("About to remove %1 file item(s).\n\n"
 				"Are you sure?")
 				.arg(items.count()),
 				QMessageBox::Ok | QMessageBox::Cancel)
@@ -729,6 +733,51 @@ void qtractorFileListView::activateItem ( QTreeWidgetItem *pItem )
 }
 
 
+// Clean-up unused file items.
+void qtractorFileListView::cleanupItem ( QTreeWidgetItem *pItem )
+{
+	if (pItem == NULL)
+		return;
+
+	switch (pItem->type()) {
+	case GroupItem: {
+		int iChildCount = pItem->childCount();
+		for (int i = 0; i < iChildCount; ++i)
+			cleanupItem(pItem->child(i));
+		break;
+	}
+	case FileItem: {
+		qtractorSession *pSession = qtractorSession::getInstance();
+		qtractorFileListItem *pFileItem
+			= static_cast<qtractorFileListItem *> (pItem);
+		if (pSession && pFileItem) {
+			const QString& sPath = pFileItem->path();
+			qtractorFileList::Item *pFileListItem
+				= pSession->files()->findItem(m_iFileType, sPath);
+			if (pFileListItem && pFileListItem->clipRefCount() < 1)
+				pItem->setSelected(true);
+		}
+		break;
+	}
+	default:
+		break;
+	}
+}
+
+
+void qtractorFileListView::cleanup (void)
+{
+	QTreeWidget::setCurrentItem(NULL);
+	QTreeWidget::selectionModel()->clearSelection();
+
+	int iItemCount = QTreeWidget::topLevelItemCount();
+	for (int i = 0; i < iItemCount; ++i)
+		cleanupItem(QTreeWidget::topLevelItem(i));
+
+	removeItem();
+}
+
+
 // Master clean-up.
 void qtractorFileListView::clear (void)
 {
@@ -751,11 +800,7 @@ qtractorFileGroupItem *qtractorFileListView::findGroupItem (
 qtractorFileListItem *qtractorFileListView::findFileItem (
 	const QString& sPath ) const
 {
-	qtractorSession *pSession = qtractorSession::getInstance();
-	if (pSession == NULL)
-		return static_cast<qtractorFileListItem *> (findItem(sPath, FileItem));
-	else
-		return pSession->files()->findFileItem(m_iFileType, sPath);
+	return static_cast<qtractorFileListItem *> (findItem(sPath, FileItem));
 }
 
 
@@ -1175,7 +1220,7 @@ void qtractorFileListView::dragMoveEvent ( QDragMoveEvent *pDragMoveEvent )
 }
 
 
-void qtractorFileListView::dragLeaveEvent ( QDragLeaveEvent * )
+void qtractorFileListView::dragLeaveEvent ( QDragLeaveEvent */*pDragLeaveEvent*/ )
 {
 	if (m_pRubberBand)
 		delete m_pRubberBand;
@@ -1389,6 +1434,10 @@ bool qtractorFileListView::loadElement ( qtractorDocument *pDocument,
 bool qtractorFileListView::saveListElement ( qtractorDocument *pDocument,
 	QDomElement *pElement, QTreeWidgetItem *pItem )
 {
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
 	// Create the new child element...
 	switch (pItem->type()) {
 	case GroupItem: {
@@ -1410,15 +1459,16 @@ bool qtractorFileListView::saveListElement ( qtractorDocument *pDocument,
 		QDomElement eFile = pDocument->document()->createElement("file");
 		eFile.setAttribute("name", pFileItem->text(0));
 		// Make it all relative to archive or session directory...
-		QString sPath;
+		QString sPath = pFileItem->path();
 		if (pDocument->isArchive()) {
-			sPath = pDocument->addFile(pFileItem->path());
+			qtractorFileList::Item *pFileListItem
+				= pSession->files()->findItem(m_iFileType, sPath);
+			if (pFileListItem && pFileListItem->clipRefCount() > 0)
+				sPath = pDocument->addFile(sPath);
 		} else {
 			QDir dir;
-			qtractorSession *pSession = qtractorSession::getInstance();
-			if (pSession)
-				dir.setPath(pSession->sessionDir());
-			sPath = dir.relativeFilePath(pFileItem->path());
+			dir.setPath(pSession->sessionDir());
+			sPath = dir.relativeFilePath(sPath);
 		}
 		eFile.appendChild(pDocument->document()->createTextNode(sPath));
 		pElement->appendChild(eFile);
