@@ -1,7 +1,7 @@
 // qtractorFiles.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2012, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -87,7 +87,7 @@ public slots:
 	// Paste file item(s) from clipboard.
 	void pasteItemSlot();
 	// Remove current group/file item(s).
-	void deleteItemSlot();
+	void removeItemSlot();
 	
 protected slots:
 
@@ -99,6 +99,8 @@ protected slots:
 	void renameItemSlot();
 	// Audition/pre-listening player slots.
 	void playSlot(bool bOn);
+	// Clean-up unused file items.
+	void cleanupSlot();
 
 	// Tab page switch slots.
 	void pageAudioSlot();
@@ -140,8 +142,9 @@ private:
 	QAction *m_pCopyItemAction;
 	QAction *m_pPasteItemAction;
 	QAction *m_pRenameItemAction;
-	QAction *m_pDeleteItemAction;
+	QAction *m_pRemoveItemAction;
 	QAction *m_pPlayItemAction;
+	QAction *m_pCleanupAction;
 };
 
 
