@@ -177,7 +177,7 @@ void qtractorInstrumentForm::importSlot (void)
 
 	const QString  sExt("ins");
 	const QString& sTitle  = tr("Import Instrument Files") + " - " QTRACTOR_TITLE;
-	const QString& sFilter = tr("Instrument files (*.%1 *.sf2)").arg(sExt);
+	const QString& sFilter = tr("Instrument files (*.%1 *.sf2 *.midnam)").arg(sExt);
 #if QT_VERSION < 0x040400
 	// Ask for the filename to open...
 	files = QFileDialog::getOpenFileNames(this,
@@ -582,7 +582,7 @@ void qtractorInstrumentForm::refreshForm (void)
 		// - Controller Names...
 		pListItem = new qtractorInstrumentGroupItem();
 		pListItem->setText(0, tr("Controller Names"));
-		listInstrumentDataList(pListItem, m_pInstruments->controllers(),
+		listInstrumentDataList(pListItem, m_pInstruments->controls(),
 			QIcon(":/images/itemControllers.png"));
 		names.append(pListItem);
 		// - RPN Names...
