@@ -24,16 +24,8 @@
 
 #include "qtractorPlugin.h"
 
-#ifdef CONFIG_LIBSLV2
-#include <slv2/slv2.h>
-#endif
-
 #ifdef CONFIG_LIBLILV
 #include <lilv/lilv.h>
-#define SLV2Plugin   LilvPlugin*
-#define SLV2Instance LilvInstance*
-#define SLV2UIs      LilvUIs*
-#define SLV2UI       LilvUI*
 #endif
 
 #ifdef CONFIG_LIBSUIL
@@ -42,8 +34,8 @@
 
 #ifdef CONFIG_LV2_EVENT
 // LV2 Event/MIDI support.
-#include "lv2_event.h"
-#include "lv2_event_helpers.h"
+#include "lv2/lv2plug.in/ns/ext/event/event.h"
+#include "lv2/lv2plug.in/ns/ext/event/event-helpers.h"
 #ifndef QTRACTOR_LV2_MIDI_EVENT_ID
 #define QTRACTOR_LV2_MIDI_EVENT_ID 1
 #endif
@@ -51,7 +43,6 @@
 
 #ifdef CONFIG_LV2_ATOM
 // LV2 Atom/MIDI support.
-#include "lv2_atom.h"
 #include "lv2_atom_helpers.h"
 #ifndef QTRACTOR_LV2_MIDI_EVENT_ID
 #define QTRACTOR_LV2_MIDI_EVENT_ID 1
@@ -60,7 +51,7 @@
 
 #ifdef CONFIG_LV2_WORKER
 // LV2 Worker/Schedule support.
-#include "lv2_worker.h"
+#include "lv2/lv2plug.in/ns/ext/worker/worker.h"
 // Forward declarations.
 class qtractorLv2Worker;
 #endif
@@ -71,10 +62,10 @@ class qtractorLv2Worker;
 
 #ifdef CONFIG_LV2_UI
 // LV2 UI support.
-#include "lv2_ui.h"
+#include "lv2/lv2plug.in/ns/extensions/ui/ui.h"
 // LV2 UI data/instance access support.
-#include "lv2_data_access.h"
-#include "lv2_instance_access.h"
+#include "lv2/lv2plug.in/ns/ext/data-access/data-access.h"
+#include "lv2/lv2plug.in/ns/ext/instance-access/instance-access.h"
 #ifdef CONFIG_LV2_ATOM
 #include <jack/ringbuffer.h>
 #endif
@@ -87,8 +78,8 @@ class qtractorLv2Worker;
 
 #ifdef CONFIG_LV2_STATE
 // LV2 State support.
-#include "lv2_atom.h"
-#include "lv2_state.h"
+#include "lv2/lv2plug.in/ns/ext/atom/atom.h"
+#include "lv2/lv2plug.in/ns/ext/state/state.h"
 #endif
 
 #ifdef CONFIG_LV2_PROGRAMS
@@ -96,21 +87,22 @@ class qtractorLv2Worker;
 #include "lv2_programs.h"
 #endif
 
-#ifdef CONFIG_LIBLILV
-
 #ifdef CONFIG_LV2_PRESETS
 // LV2 Presets support.
-#include "lv2_presets.h"
+#include "lv2/lv2plug.in/ns/ext/presets/presets.h"
 #endif
 
 #ifdef CONFIG_LV2_TIME
 // LV2 Time support.
-#include "lv2_time.h"
+#include "lv2/lv2plug.in/ns/ext/time/time.h"
 // JACK Transport position support.
 #include <jack/transport.h>
 #endif
 
-#endif	// CONFIG_LIBLILV
+#ifdef CONFIG_LV2_OPTIONS
+// LV2 Options support.
+#include "lv2/lv2plug.in/ns/ext/options/options.h"
+#endif	// CONFIG_LV2_OPTIONS
 
 
 //----------------------------------------------------------------------------
@@ -122,9 +114,9 @@ class qtractorLv2PluginType : public qtractorPluginType
 public:
 
 	// Constructor.
-	qtractorLv2PluginType(const QString& sUri, SLV2Plugin plugin = NULL)
+	qtractorLv2PluginType(const QString& sUri, LilvPlugin *plugin = NULL)
 		: qtractorPluginType(NULL, 0, qtractorPluginType::Lv2),
-			m_sUri(sUri), m_slv2_plugin(plugin)	{}
+			m_sUri(sUri), m_lv2_plugin(plugin)	{}
 
 	// Destructor.
 	~qtractorLv2PluginType()
@@ -136,22 +128,22 @@ public:
 
 	// Factory method (static)
 	static qtractorLv2PluginType *createType(
-		const QString& sUri, SLV2Plugin plugin = NULL);
+		const QString& sUri, LilvPlugin *plugin = NULL);
 
 	// LV2 plugin URI (virtual override).
 	QString filename() const
 		{ return m_sUri; }
 
 	// LV2 descriptor method (static)
-	static SLV2Plugin slv2_plugin(const QString& sUri);
+	static LilvPlugin *lv2_plugin(const QString& sUri);
 
 	// Specific accessors.
-	SLV2Plugin slv2_plugin() const
-		{ return m_slv2_plugin; }
+	LilvPlugin *lv2_plugin() const
+		{ return m_lv2_plugin; }
 
 	// LV2 World stuff (ref. counted).
-	static void slv2_open();
-	static void slv2_close();
+	static void lv2_open();
+	static void lv2_close();
 
 	// Plugin type listing (static).
 	static bool getTypes(qtractorPluginPath& path);
@@ -171,7 +163,7 @@ protected:
 	QString    m_sUri;
 
 	// LV2 descriptor itself.
-	SLV2Plugin m_slv2_plugin;
+	LilvPlugin *m_lv2_plugin;
 
 #ifdef CONFIG_LV2_EVENT
 	unsigned short m_iMidiEventIns;
@@ -210,8 +202,8 @@ public:
 	void process(float **ppIBuffer, float **ppOBuffer, unsigned int nframes);
 
 	// Specific accessors.
-	SLV2Plugin slv2_plugin() const;
-	SLV2Instance slv2_instance(unsigned short iInstance) const;
+	LilvPlugin *lv2_plugin() const;
+	LilvInstance *lv2_instance(unsigned short iInstance) const;
 
 	LV2_Handle lv2_handle(unsigned short iInstance) const;
 
@@ -240,19 +232,10 @@ public:
 	// Idle editor (static).
 	static void idleEditorAll();
 
-	// LV2 UI descriptor accessor.
-	const LV2UI_Descriptor *lv2_ui_descriptor() const;
-
-	// LV2 UI handle accessor.
-	LV2UI_Handle lv2_ui_handle() const;
-
 	// LV2 UI control change method.
 	void lv2_ui_write(uint32_t port_index,
 		uint32_t buffer_size, uint32_t protocol, const void *buffer);
 
-	// LV2 UI cleanup method.
-	void lv2_ui_cleanup() const;
-
 	// GUI editor closed state.
 	void setEditorClosed(bool bClosed)
 		{ m_bEditorClosed = bClosed; }
@@ -298,11 +281,6 @@ public:
 	// LV2 Programs extension data descriptor accessor.
 	const LV2_Programs_Interface *lv2_programs_descriptor(unsigned short iInstance) const;
 
-#ifdef CONFIG_LV2_UI
-	// LV2 Programs UI descriptor accessor.
-	const LV2_Programs_UI_Interface *lv2_ui_programs_descriptor (void) const;
-#endif
-
 	// Bank/program selector override.
 	void selectProgram(int iBank, int iProg);
 
@@ -311,25 +289,27 @@ public:
 
 #endif
 
-#ifdef CONFIG_LIBLILV
 #ifdef CONFIG_LV2_PRESETS
 	// Refresh and load preset labels listing.
 	QStringList presetList() const;
 	// Load/Save plugin state from/into a named preset.
 	bool loadPreset(const QString& sPreset);
 	bool savePreset(const QString& sPreset);
+	// Delete plugin state preset (from file-system).
+	bool deletePreset(const QString& sPreset);
+	// Whether given preset is internal/read-only.
+	bool isReadOnlyPreset(const QString& sPreset) const;
 #endif
 #ifdef CONFIG_LV2_TIME
 	// Update LV2 Time from JACK transport position.
 	static void updateTime(
 		const jack_transport_state_t state, const jack_position_t *pPos);
 #endif
-#endif	// CONFIG_LIBLILV
 
 private:
 
 	// Instance variables.
-	SLV2Instance  *m_pInstances;
+	LilvInstance **m_ppInstances;
 
 	// List of output control port indexes and data.
 	unsigned long *m_piControlOuts;
@@ -365,7 +345,7 @@ private:
 
 #ifdef CONFIG_LV2_WORKER
 	// LV2 Worker/Schedule support.
-	qtractorLv2Worker **m_lv2_workers;
+	qtractorLv2Worker *m_lv2_worker;
 #endif
 
 #ifdef CONFIG_LV2_UI
@@ -377,8 +357,8 @@ private:
 
 	volatile bool  m_bEditorClosed;
 
-	SLV2UIs        m_slv2_uis;
-	SLV2UI         m_slv2_ui;
+	LilvUIs       *m_lv2_uis;
+	LilvUI        *m_lv2_ui;
 
 	LV2_Extension_Data_Feature m_lv2_data_access;
 
@@ -393,11 +373,6 @@ private:
 	SuilWidget     m_lv2_ui_widget;
 #endif
 
-#if CONFIG_LIBSLV2
-	SLV2UIInstance m_slv2_ui_instance;
-	LV2UI_Widget   m_lv2_ui_widget;
-#endif
-	
 #ifdef CONFIG_LV2_ATOM
 
 	// LV2 Atom control (ring)buffers for UI updates.
@@ -416,17 +391,10 @@ private:
 
 #ifdef CONFIG_LV2_EXTERNAL_UI
 	LV2_Feature          m_lv2_ui_external_feature;
-	lv2_external_ui_host m_lv2_ui_external_host;
+	LV2_External_UI_Host m_lv2_ui_external_host;
 #ifdef LV2_EXTERNAL_UI_DEPRECATED_URI
 	LV2_Feature          m_lv2_ui_external_deprecated_feature;
-	lv2_external_ui_host m_lv2_ui_external_deprecated_host;
-#endif
-#endif
-
-#ifdef CONFIG_LIBSLV2
-#ifdef CONFIG_LV2_GTK_UI
-	// Our own GTK UI widget (parent frame).
-	struct _GtkWidget *m_pGtkWindow;
+	LV2_External_UI_Host m_lv2_ui_external_deprecated_host;
 #endif
 #endif
 
@@ -437,6 +405,9 @@ private:
 	QWidget     *m_pQt4Widget;
 #endif
 
+	// Changed UI params hash-queue.
+	QHash<uint32_t, float> m_ui_params;
+
 #endif	// CONFIG_LV2_UI
 
 #ifdef CONFIG_LV2_STATE
@@ -447,20 +418,31 @@ private:
 #ifdef CONFIG_LV2_STATE_FILES
 	LV2_Feature                m_lv2_state_map_path_feature;
 	LV2_State_Map_Path         m_lv2_state_map_path;
+#ifdef CONFIG_LV2_STATE_MAKE_PATH
 	LV2_Feature                m_lv2_state_make_path_feature;
 	LV2_State_Make_Path        m_lv2_state_make_path;
 #endif
+#endif
 
-#ifdef CONFIG_LIBLILV
 #ifdef CONFIG_LV2_PRESETS
 	// LV2 Presets label-to-uri map.
 	QHash<QString, QString> m_lv2_presets;
 #endif
+
 #ifdef CONFIG_LV2_TIME
 	// LV2 Time designated ports map.
 	QHash<int, unsigned long>  m_lv2_time_ports;
 #endif
-#endif	// CONFIG_LIBLILV
+
+#ifdef CONFIG_LV2_OPTIONS
+#ifdef CONFIG_LV2_BUF_SIZE
+	LV2_Feature        m_lv2_options_feature;
+	LV2_Options_Option m_lv2_options[4];
+	uint32_t           m_iMinBlockLength;
+	uint32_t           m_iMaxBlockLength;
+	uint32_t           m_iSequenceSize;
+#endif
+#endif
 };
 
 
