@@ -141,6 +141,8 @@ public slots:
 	void editSelectTrack();
 	void editSelectTrackRange();
 	void editSelectRange();
+	void editInsertRange();
+	void editInsertTrackRange();
 
 	void trackAdd();
 	void trackRemove();
@@ -359,7 +361,9 @@ protected:
 
 	void setSongPos(unsigned short iSongPos);
 
-	void updateSession();
+	void updateSessionPre();
+	void updateSessionPost();
+
 	void updateRecentFiles(const QString& sFilename);
 	void updatePeakAutoRemove();
 	void updateMessagesFont();
@@ -453,6 +457,9 @@ private:
 
 	QPalette *m_paletteItems[PaletteItems];
 
+	// View/Snap-to-beat actions (for shortcuts access)
+	QList<QAction *> m_snapPerBeatActions;
+
 	// Name says it all...
 	QList<qtractorMidiEditorForm *> m_editors;
 
