@@ -233,7 +233,7 @@ qtractorMidiManager::qtractorMidiManager (
 	m_iPendingBankLSB(-1),
 	m_iPendingProg(-1)
 {
-	const unsigned int MaxMidiEvents = (bufferSize() << 1);
+	const unsigned int MaxMidiEvents = (bufferSize() >> 1);
 
 	m_pBuffer = new snd_seq_event_t [MaxMidiEvents];
 
@@ -248,8 +248,12 @@ qtractorMidiManager::qtractorMidiManager (
 		+ MaxMidiEvents * sizeof(VstMidiEvent *);
 #endif
 #ifdef CONFIG_LV2_EVENT
-	const unsigned int Lv2BufferSize
+	const unsigned int Lv2EventBufferSize
 		= (sizeof(LV2_Event) + 4) * MaxMidiEvents;
+#endif
+#ifdef CONFIG_LV2_ATOM
+	const unsigned int Lv2AtomBufferSize
+		= (sizeof(LV2_Atom_Event) + 4) * MaxMidiEvents;
 #endif
 	for (unsigned short i = 0; i < 2; ++i) {
 	#ifdef CONFIG_VST
@@ -257,9 +261,13 @@ qtractorMidiManager::qtractorMidiManager (
 		m_ppVstMidiBuffers[i] = new VstMidiEvent [MaxMidiEvents];
 	#endif
 	#ifdef CONFIG_LV2_EVENT
-		m_ppLv2Buffers[i] = lv2_event_buffer_new(Lv2BufferSize,
+		m_ppLv2EventBuffers[i] = lv2_event_buffer_new(Lv2EventBufferSize,
 			LV2_EVENT_AUDIO_STAMP);
 	#endif
+	#ifdef CONFIG_LV2_ATOM
+		m_ppLv2AtomBuffers[i] = lv2_atom_buffer_new(Lv2AtomBufferSize,
+			qtractorLv2Plugin::lv2_urid_map(LV2_ATOM__Sequence), (i & 1) == 0);
+	#endif
 	}
 
 	m_bWaitSync = false;
@@ -296,7 +304,10 @@ qtractorMidiManager::~qtractorMidiManager (void)
 		delete [] m_ppVstBuffers[i];
 	#endif
 	#ifdef CONFIG_LV2_EVENT
-		::free(m_ppLv2Buffers[i]);
+		::free(m_ppLv2EventBuffers[i]);
+	#endif
+	#ifdef CONFIG_LV2_ATOM
+		lv2_atom_buffer_free(m_ppLv2AtomBuffers[i]);
 	#endif
 	}
 
@@ -334,30 +345,37 @@ bool qtractorMidiManager::direct ( snd_seq_event_t *pEvent )
 	return m_directBuffer.push(pEvent);
 }
 
+#include "qtractorMidiEngine.h"
 
 // Queued buffering.
-bool qtractorMidiManager::queued (
-	qtractorTimeScale *pTimeScale, snd_seq_event_t *pEvent )
+bool qtractorMidiManager::queued ( qtractorTimeScale *pTimeScale,
+	snd_seq_event_t *pEvent, unsigned long iTime, long iFrameStart )
 {
-	unsigned long iTick = pTimeScale->frameFromTick(pEvent->time.tick);
+	qtractorTimeScale::Cursor& cursor = pTimeScale->cursor();
+	qtractorTimeScale::Node *pNode = cursor.seekTick(iTime);
+	unsigned long t0 = pNode->frameFromTick(iTime);
+	unsigned long t1 = (long(t0) > iFrameStart ? t0 - iFrameStart : t0);
 
 	if (pEvent->type == SND_SEQ_EVENT_NOTE) {
 		snd_seq_event_t ev = *pEvent;
 		ev.type = SND_SEQ_EVENT_NOTEON;
-		if (!m_queuedBuffer.insert(&ev, iTick))
+		if (!m_queuedBuffer.insert(&ev, t1))
 			return false;
-		if (ev.data.note.duration > 0)
-			iTick += pTimeScale->frameFromTick(ev.data.note.duration - 1);
+		if (ev.data.note.duration > 0) {
+			iTime += (ev.data.note.duration - 1);
+			pNode = cursor.seekTick(iTime);
+			t1 += (pNode->frameFromTick(iTime) - t0);
+		}
 		ev.type = SND_SEQ_EVENT_NOTEOFF;
 		ev.data.note.velocity = 0;
 		ev.data.note.duration = 0;
-		return m_postedBuffer.insert(&ev, iTick);
+		return m_postedBuffer.insert(&ev, t1);
 	}
 
 	if (pEvent->type == SND_SEQ_EVENT_NOTEOFF)
-		return m_postedBuffer.insert(pEvent, iTick);
+		return m_postedBuffer.insert(pEvent, t1);
 	else
-		return m_queuedBuffer.insert(pEvent, iTick);
+		return m_queuedBuffer.insert(pEvent, t1);
 }
 
 
@@ -373,9 +391,12 @@ void qtractorMidiManager::clear (void)
 		::memset(pVstEvents, 0, sizeof(VstEvents));
 	#endif
 	#ifdef CONFIG_LV2_EVENT
-		LV2_Event_Buffer *pLv2Buffer = m_ppLv2Buffers[i];
-		lv2_event_buffer_reset(pLv2Buffer, LV2_EVENT_AUDIO_STAMP,
-			(unsigned char *) (pLv2Buffer + 1));
+		LV2_Event_Buffer *pLv2EventBuffer = m_ppLv2EventBuffers[i];
+		lv2_event_buffer_reset(pLv2EventBuffer, LV2_EVENT_AUDIO_STAMP,
+			(unsigned char *) (pLv2EventBuffer + 1));
+	#endif
+	#ifdef CONFIG_LV2_ATOM
+		lv2_atom_buffer_reset(m_ppLv2AtomBuffers[i], (i & 1) == 0);
 	#endif
 	}
 
@@ -452,9 +473,14 @@ void qtractorMidiManager::process (
 		VstEvents *pVstEvents = (VstEvents *) m_ppVstBuffers[m_iEventBuffer & 1];
 	#endif
 	#ifdef CONFIG_LV2_EVENT
-		LV2_Event_Buffer *pLv2Buffer = m_ppLv2Buffers[m_iEventBuffer & 1];
-		LV2_Event_Iterator iter;
-		lv2_event_begin(&iter, pLv2Buffer);
+		LV2_Event_Buffer *pLv2EventBuffer = m_ppLv2EventBuffers[m_iEventBuffer & 1];
+		LV2_Event_Iterator eiter;
+		lv2_event_begin(&eiter, pLv2EventBuffer);
+	#endif
+	#ifdef CONFIG_LV2_ATOM
+		LV2_Atom_Buffer *pLv2AtomBuffer = m_ppLv2AtomBuffers[m_iEventBuffer & 1];
+		LV2_Atom_Buffer_Iterator aiter;
+		lv2_atom_buffer_begin(&aiter, pLv2AtomBuffer);
 	#endif
 		const unsigned int MaxMidiEvents = (bufferSize() << 1);
 		unsigned int iMidiEvents = 0;
@@ -483,7 +509,11 @@ void qtractorMidiManager::process (
 			fprintf(stderr, " }\n");
 		#endif
 		#ifdef CONFIG_LV2_EVENT
-			lv2_event_write(&iter, pEv->time.tick, 0,
+			lv2_event_write(&eiter, pEv->time.tick, 0,
+				QTRACTOR_LV2_MIDI_EVENT_ID, iMidiData, pMidiData);
+		#endif
+		#ifdef CONFIG_LV2_ATOM
+			lv2_atom_buffer_write(&aiter, pEv->time.tick, 0,
 				QTRACTOR_LV2_MIDI_EVENT_ID, iMidiData, pMidiData);
 		#endif
 		#ifdef CONFIG_VST
@@ -665,9 +695,12 @@ void qtractorMidiManager::swapEventBuffers (void)
 	::memset(m_ppVstBuffers[m_iEventBuffer & 1], 0, sizeof(VstEvents));
 #endif
 #ifdef CONFIG_LV2_EVENT
-	LV2_Event_Buffer *pLv2Buffer = m_ppLv2Buffers[m_iEventBuffer & 1];
-	lv2_event_buffer_reset(pLv2Buffer, LV2_EVENT_AUDIO_STAMP,
-		(unsigned char *) (pLv2Buffer + 1));
+	LV2_Event_Buffer *pLv2EventBuffer = m_ppLv2EventBuffers[m_iEventBuffer & 1];
+	lv2_event_buffer_reset(pLv2EventBuffer, LV2_EVENT_AUDIO_STAMP,
+		(unsigned char *) (pLv2EventBuffer + 1));
+#endif
+#ifdef CONFIG_LV2_ATOM
+	lv2_atom_buffer_reset(m_ppLv2AtomBuffers[m_iEventBuffer & 1], false);
 #endif
 
 	++m_iEventBuffer;
@@ -704,11 +737,17 @@ void qtractorMidiManager::vst_events_swap (void)
 	VstMidiEvent *pVstMidiBuffer = m_ppVstMidiBuffers[iEventBuffer];
 	VstEvents *pVstEvents = (VstEvents *) m_ppVstBuffers[iEventBuffer];
 #ifdef CONFIG_LV2_EVENT
-	LV2_Event_Buffer *pLv2Buffer = m_ppLv2Buffers[iEventBuffer];
-	lv2_event_buffer_reset(pLv2Buffer, LV2_EVENT_AUDIO_STAMP,
-		(unsigned char *) (pLv2Buffer + 1));
-	LV2_Event_Iterator iter;
-	lv2_event_begin(&iter, pLv2Buffer);
+	LV2_Event_Buffer *pLv2EventBuffer = m_ppLv2EventBuffers[iEventBuffer];
+	lv2_event_buffer_reset(pLv2EventBuffer, LV2_EVENT_AUDIO_STAMP,
+		(unsigned char *) (pLv2EventBuffer + 1));
+	LV2_Event_Iterator eiter;
+	lv2_event_begin(&eiter, pLv2EventBuffer);
+#endif
+#ifdef CONFIG_LV2_ATOM
+	LV2_Atom_Buffer *pLv2AtomBuffer = m_ppLv2AtomBuffers[iEventBuffer];
+	lv2_atom_buffer_reset(pLv2AtomBuffer, true);
+	LV2_Atom_Buffer_Iterator aiter;
+	lv2_atom_buffer_begin(&aiter, pLv2AtomBuffer);
 #endif
 	unsigned int iMidiEvents = 0;
 	const unsigned int MaxMidiEvents = (bufferSize() << 1);
@@ -729,7 +768,11 @@ void qtractorMidiManager::vst_events_swap (void)
 		}
 	#endif
 	#ifdef CONFIG_LV2_EVENT
-		lv2_event_write(&iter, pVstMidiEvent->deltaFrames, 0,
+		lv2_event_write(&eiter, pVstMidiEvent->deltaFrames, 0,
+			QTRACTOR_LV2_MIDI_EVENT_ID, iMidiData, pMidiData);
+	#endif
+	#ifdef CONFIG_LV2_ATOM
+		lv2_atom_buffer_write(&aiter, pVstMidiEvent->deltaFrames, 0,
 			QTRACTOR_LV2_MIDI_EVENT_ID, iMidiData, pMidiData);
 	#endif
 		++iMidiEvents;
@@ -747,51 +790,63 @@ void qtractorMidiManager::vst_events_swap (void)
 void qtractorMidiManager::lv2_events_swap (void)
 {
 	unsigned short iEventBuffer = (m_iEventBuffer + 1) & 1;
-	LV2_Event_Buffer *pLv2Buffer = m_ppLv2Buffers[iEventBuffer];
+	LV2_Event_Buffer *pLv2EventBuffer = m_ppLv2EventBuffers[iEventBuffer];
 #ifdef CONFIG_VST
 	VstMidiEvent *pVstMidiBuffer = m_ppVstMidiBuffers[iEventBuffer];
 	VstEvents *pVstEvents = (VstEvents *) m_ppVstBuffers[iEventBuffer];
 	::memset(pVstEvents, 0, sizeof(VstEvents));
 #endif
-	LV2_Event_Iterator iter;
-	lv2_event_begin(&iter, pLv2Buffer);
+#ifdef CONFIG_LV2_ATOM
+	LV2_Atom_Buffer *pLv2AtomBuffer = m_ppLv2AtomBuffers[iEventBuffer];
+	lv2_atom_buffer_reset(pLv2AtomBuffer, true);
+	LV2_Atom_Buffer_Iterator aiter;
+	lv2_atom_buffer_begin(&aiter, pLv2AtomBuffer);
+#endif
+	LV2_Event_Iterator eiter;
+	lv2_event_begin(&eiter, pLv2EventBuffer);
 	unsigned int iMidiEvents = 0;
 	const unsigned int MaxMidiEvents = (bufferSize() << 1);
 	while (iMidiEvents < MaxMidiEvents
-			&& lv2_event_is_valid(&iter)) {
+			&& lv2_event_is_valid(&eiter)) {
 		unsigned char *pMidiData;
-		LV2_Event *pLv2Event = lv2_event_get(&iter, &pMidiData);
+		LV2_Event *pLv2Event = lv2_event_get(&eiter, &pMidiData);
 		if (pLv2Event == NULL)
 			break;
-		long iMidiData = pLv2Event->size;
-		if (iMidiData < 1)
-			break;
-	#ifdef CONFIG_VST
-		VstMidiEvent *pVstMidiEvent = &pVstMidiBuffer[iMidiEvents];
-		if (iMidiData >= long(sizeof(pVstMidiEvent->midiData)))
-			break;
-	#endif
-	#ifdef CONFIG_MIDI_PARSER
-		if (m_pMidiParser) {
-			snd_seq_event_t *pEv = &m_pBuffer[iMidiEvents];
-		//	snd_seq_ev_clear(pEv);
-			iMidiData = snd_midi_event_encode(m_pMidiParser,
-				pMidiData, iMidiData, pEv);
-			if (iMidiData < 1 || pEv->type == SND_SEQ_EVENT_NONE)
+		if (pLv2Event->type == QTRACTOR_LV2_MIDI_EVENT_ID) {
+			long iMidiData = pLv2Event->size;
+			if (iMidiData < 1)
 				break;
-			pEv->time.tick = pLv2Event->frames;
+		#ifdef CONFIG_VST
+			VstMidiEvent *pVstMidiEvent = &pVstMidiBuffer[iMidiEvents];
+			if (iMidiData >= long(sizeof(pVstMidiEvent->midiData)))
+				break;
+		#endif
+		#ifdef CONFIG_MIDI_PARSER
+			if (m_pMidiParser) {
+				snd_seq_event_t *pEv = &m_pBuffer[iMidiEvents];
+			//	snd_seq_ev_clear(pEv);
+				iMidiData = snd_midi_event_encode(m_pMidiParser,
+					pMidiData, iMidiData, pEv);
+				if (iMidiData < 1 || pEv->type == SND_SEQ_EVENT_NONE)
+					break;
+				pEv->time.tick = pLv2Event->frames;
+			}
+		#endif
+		#ifdef CONFIG_VST
+			::memset(pVstMidiEvent, 0, sizeof(VstMidiEvent));
+			pVstMidiEvent->type = kVstMidiType;
+			pVstMidiEvent->byteSize = sizeof(VstMidiEvent);
+			pVstMidiEvent->deltaFrames = pLv2Event->frames;
+			::memcpy(&pVstMidiEvent->midiData[0], pMidiData, iMidiData);
+			pVstEvents->events[iMidiEvents] = (VstEvent *) pVstMidiEvent;
+		#endif
+		#ifdef CONFIG_LV2_ATOM
+			lv2_atom_buffer_write(&aiter, pLv2Event->frames, 0,
+				QTRACTOR_LV2_MIDI_EVENT_ID, iMidiData, pMidiData);
+		#endif
+			++iMidiEvents;
 		}
-	#endif
-	#ifdef CONFIG_VST
-		::memset(pVstMidiEvent, 0, sizeof(VstMidiEvent));
-		pVstMidiEvent->type = kVstMidiType;
-		pVstMidiEvent->byteSize = sizeof(VstMidiEvent);
-		pVstMidiEvent->deltaFrames = pLv2Event->frames;
-		::memcpy(&pVstMidiEvent->midiData[0], pMidiData, iMidiData);
-		pVstEvents->events[iMidiEvents] = (VstEvent *) pVstMidiEvent;
-	#endif
-		lv2_event_increment(&iter);
-		++iMidiEvents;
+		lv2_event_increment(&eiter);
 	}
 #ifdef CONFIG_VST
 	pVstEvents->numEvents = iMidiEvents;
@@ -803,6 +858,80 @@ void qtractorMidiManager::lv2_events_swap (void)
 #endif	// CONFIG_LV2_EVENT
 
 
+#ifdef CONFIG_LV2_ATOM
+
+// Swap LV2 atom buffers...
+void qtractorMidiManager::lv2_atom_buffer_swap (void)
+{
+	unsigned short iEventBuffer = (m_iEventBuffer + 1) & 1;
+	LV2_Atom_Buffer *pLv2AtomBuffer = m_ppLv2AtomBuffers[iEventBuffer];
+#ifdef CONFIG_VST
+	VstMidiEvent *pVstMidiBuffer = m_ppVstMidiBuffers[iEventBuffer];
+	VstEvents *pVstEvents = (VstEvents *) m_ppVstBuffers[iEventBuffer];
+	::memset(pVstEvents, 0, sizeof(VstEvents));
+#endif
+#ifdef CONFIG_LV2_EVENT
+	LV2_Event_Buffer *pLv2EventBuffer = m_ppLv2EventBuffers[iEventBuffer];
+	lv2_event_buffer_reset(pLv2EventBuffer, LV2_EVENT_AUDIO_STAMP,
+		(unsigned char *) (pLv2EventBuffer + 1));
+	LV2_Event_Iterator eiter;
+	lv2_event_begin(&eiter, pLv2EventBuffer);
+#endif
+	LV2_Atom_Buffer_Iterator aiter;
+	lv2_atom_buffer_begin(&aiter, pLv2AtomBuffer);
+	unsigned int iMidiEvents = 0;
+	const unsigned int MaxMidiEvents = (bufferSize() << 1);
+	while (iMidiEvents < MaxMidiEvents) {
+		unsigned char *pMidiData;
+		LV2_Atom_Event *pLv2AtomEvent = lv2_atom_buffer_get(&aiter, &pMidiData);
+		if (pLv2AtomEvent == NULL)
+			break;
+		if (pLv2AtomEvent->body.type == QTRACTOR_LV2_MIDI_EVENT_ID) {
+			long iMidiData = pLv2AtomEvent->body.size;
+			if (iMidiData < 1)
+				break;
+		#ifdef CONFIG_VST
+			VstMidiEvent *pVstMidiEvent = &pVstMidiBuffer[iMidiEvents];
+			if (iMidiData >= long(sizeof(pVstMidiEvent->midiData)))
+				break;
+		#endif
+		#ifdef CONFIG_MIDI_PARSER
+			if (m_pMidiParser) {
+				snd_seq_event_t *pEv = &m_pBuffer[iMidiEvents];
+			//	snd_seq_ev_clear(pEv);
+				iMidiData = snd_midi_event_encode(m_pMidiParser,
+					pMidiData, iMidiData, pEv);
+				if (iMidiData < 1 || pEv->type == SND_SEQ_EVENT_NONE)
+					break;
+				pEv->time.tick = pLv2AtomEvent->time.frames;
+			}
+		#endif
+		#ifdef CONFIG_VST
+			::memset(pVstMidiEvent, 0, sizeof(VstMidiEvent));
+			pVstMidiEvent->type = kVstMidiType;
+			pVstMidiEvent->byteSize = sizeof(VstMidiEvent);
+			pVstMidiEvent->deltaFrames = pLv2AtomEvent->time.frames;
+			::memcpy(&pVstMidiEvent->midiData[0], pMidiData, iMidiData);
+			pVstEvents->events[iMidiEvents] = (VstEvent *) pVstMidiEvent;
+		#endif
+		#ifdef CONFIG_LV2_EVENT
+			lv2_event_write(&eiter, pLv2AtomEvent->time.frames, 0,
+				QTRACTOR_LV2_MIDI_EVENT_ID, iMidiData, pMidiData);
+		#endif
+			++iMidiEvents;
+		}
+		lv2_atom_buffer_increment(&aiter);
+	}
+#ifdef CONFIG_VST
+	pVstEvents->numEvents = iMidiEvents;
+#endif
+	m_iBuffer = iMidiEvents;
+	swapEventBuffers();
+}
+
+#endif	// CONFIG_LV2_ATOM
+
+
 // Some default factory options.
 void qtractorMidiManager::setDefaultAudioOutputBus ( bool bAudioOutputBus )
 {
