@@ -1,7 +1,7 @@
 // qtractorMidiControlObserverForm.h
 //
 /****************************************************************************
-   Copyright (C) 2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,6 +28,8 @@
 class qtractorMidiControlObserver;
 class qtractorCtlEvent;
 
+class QCloseEvent;
+
 
 //----------------------------------------------------------------------------
 // qtractorMidiControlObserverForm -- UI wrapper form.
@@ -38,15 +40,13 @@ class qtractorMidiControlObserverForm : public QDialog
 
 public:
 
-	// Constructor.
-	qtractorMidiControlObserverForm(QWidget *pParent = 0, Qt::WindowFlags wflags = 0);
-
-	// Destructor.
-	~qtractorMidiControlObserverForm();
-
 	// Pseudo-singleton instance.
 	static qtractorMidiControlObserverForm *getInstance();
 
+	// Pseudo-constructor.
+	static void showInstance(qtractorMidiControlObserver *pMidiObserver,
+		QWidget *pParent = 0, Qt::WindowFlags wflags = 0);
+
 	// Observer accessors.
 	void setMidiObserver(qtractorMidiControlObserver *pMidiObserver);
 	qtractorMidiControlObserver *midiObserver() const;
@@ -54,6 +54,12 @@ public:
 	// Process incoming controller event.
 	void processEvent(const qtractorCtlEvent& ctle);
 
+	// MIDI controller/observer attachment (context menu) activator.
+	static QAction *addMidiControlAction(QWidget *pParent,
+		QWidget *pWidget, qtractorMidiControlObserver *pMidiObserver);
+	static void midiControlAction(QWidget *pParent, QAction *pAction);
+	static void midiControlMenu(QWidget *pParent, const QPoint& pos);
+
 protected slots:
 
 	void activateControlType(const QString&);
@@ -64,8 +70,24 @@ protected slots:
 
 	void accept();
 	void reject();
+	void inputs();
+	void outputs();
 	void reset();
 
+	void stabilizeForm();
+
+protected:
+
+	// Constructor.
+	qtractorMidiControlObserverForm(QWidget *pParent = 0, Qt::WindowFlags wflags = 0);
+
+	// Pseudo-destructor.
+	void closeEvent(QCloseEvent *pCloseEvent);
+
+	// Add esquisite automation menu actions...
+	static void addMidiControlMenu(
+		QMenu *pMenu, qtractorMidiControlObserver *pMidiObserver);
+
 private:
 
 	// The Qt-designer UI struct...
