@@ -214,7 +214,7 @@ void qtractorMidiEditEvent::setEventType (
 {
 	m_eventType = eventType;
 
-	m_pEditor->selectAll(false);
+	m_pEditor->selectAll(this, false);
 //	m_pEditor->updateContents();
 }
 
@@ -228,7 +228,7 @@ void qtractorMidiEditEvent::setController ( unsigned char controller )
 {
 	m_controller = controller;
 
-	m_pEditor->selectAll(false);
+	m_pEditor->selectAll(this, false);
 //	m_pEditor->updateContents();
 }
 
@@ -275,8 +275,8 @@ void qtractorMidiEditEvent::resizeEvent ( QResizeEvent *pResizeEvent )
 void qtractorMidiEditEvent::updatePixmap ( int cx, int /*cy*/ )
 {
 	QWidget *pViewport = qtractorScrollView::viewport();
-	int w = pViewport->width();
-	int h = pViewport->height();
+	const int w = pViewport->width();
+	const int h = pViewport->height() & ~1; // always even.
 
 	if (w < 1 || h < 1)
 		return;
@@ -292,6 +292,10 @@ void qtractorMidiEditEvent::updatePixmap ( int cx, int /*cy*/ )
 	m_pixmap = QPixmap(w, h);
 	m_pixmap.fill(rgbBase);
 
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
 	qtractorTimeScale *pTimeScale = m_pEditor->timeScale();
 	if (pTimeScale == NULL)
 		return;
@@ -336,27 +340,25 @@ void qtractorMidiEditEvent::updatePixmap ( int cx, int /*cy*/ )
 	int x = pNode->pixelFromBeat(iBeat) - dx;
 	int x2 = x;
 	while (x < w) {
-		if (x >= 0) {
-			bool bBeatIsBar = pNode->beatIsBar(iBeat);
-			if (bBeatIsBar) {
-				p.setPen(rgbLight);
-				p.drawLine(x, 0, x, h);
-				if (m_pEditor->isSnapZebra() && (x > x2) && (++iBar & 1))
-					p.fillRect(QRect(x2, 0, x - x2 + 1, h), zebra);
-				x2 = x;
-				if (iBeat == pNode->beat)
-					iPixelsPerBeat = pNode->pixelsPerBeat();
-			}
-			if (bBeatIsBar || iPixelsPerBeat > 8) {
-				p.setPen(rgbDark);
-				p.drawLine(x - 1, 0, x - 1, h);
-			}
+		bool bBeatIsBar = pNode->beatIsBar(iBeat);
+		if (bBeatIsBar) {
+			p.setPen(rgbDark);
+			p.drawLine(x - 1, 0, x - 1, h);
+			if (m_pEditor->isSnapZebra() && (x > x2) && (++iBar & 1))
+				p.fillRect(QRect(x2, 0, x - x2 + 1, h), zebra);
+			x2 = x;
+			if (iBeat == pNode->beat)
+				iPixelsPerBeat = pNode->pixelsPerBeat();
+		}
+		if (bBeatIsBar || iPixelsPerBeat > 8) {
+			p.setPen(rgbLight);
+			p.drawLine(x, 0, x, h);
 		}
 		if (iSnapPerBeat > 1) {
 			int q = iPixelsPerBeat / iSnapPerBeat;
 			if (q > 4) {  
 				p.setPen(rgbBase.value() < 0x7f
-					? rgbDark.darker(105) : rgbLight.lighter(120));
+					? rgbLight.darker(105) : rgbLight.lighter(120));
 				for (int i = 1; i < iSnapPerBeat; ++i) {
 					x = pTimeScale->pixelSnap(x + dx + q) - dx - 1;
 					p.drawLine(x, 0, x, h);
@@ -369,6 +371,17 @@ void qtractorMidiEditEvent::updatePixmap ( int cx, int /*cy*/ )
 	if (m_pEditor->isSnapZebra() && (x > x2) && (++iBar & 1))
 		p.fillRect(QRect(x2, 0, x - x2 + 1, h), zebra);
 
+	// Draw location marker lines...
+	qtractorTimeScale::Marker *pMarker
+		= pTimeScale->markers().seekPixel(dx);
+	while (pMarker) {
+		x = pTimeScale->pixelFromFrame(pMarker->frame) - dx;
+		if (x > w) break;
+		p.setPen(pMarker->color);
+		p.drawLine(x, 0, x, h);
+		pMarker = pMarker->next();
+	}
+
 	//
 	// Draw the sequence events...
 	//
@@ -383,9 +396,7 @@ void qtractorMidiEditEvent::updatePixmap ( int cx, int /*cy*/ )
 	unsigned long iTickEnd = pNode->tickFromPixel(x);
 
 	// This is the zero-line...
-	int y0 = h;
-	if (m_eventType == qtractorMidiEvent::PITCHBEND)
-		y0 = (dy << 2);
+	const int y0 = (m_eventType == qtractorMidiEvent::PITCHBEND ? h >> 1 : h);
 
 	p.setPen(rgbLight);
 	p.drawLine(0, y0 - 1, w, y0 - 1);
@@ -445,10 +456,6 @@ void qtractorMidiEditEvent::updatePixmap ( int cx, int /*cy*/ )
 		pEvent = pEvent->next();
 	}
 
-	qtractorSession *pSession = qtractorSession::getInstance();
-	if (pSession == NULL)
-		return;
-
 	// Draw loop boundaries, if applicable...
 	if (pSession->isLooping()) {
 		const QBrush shade(QColor(0, 0, 0, 60));
@@ -502,13 +509,27 @@ void qtractorMidiEditEvent::drawContents ( QPainter *pPainter, const QRect& rect
 
 	m_pEditor->paintDragState(this, pPainter);
 
-	// Draw special play-head header...
+	// Draw special play-head line...
 	int cx = qtractorScrollView::contentsX();
 	int x  = m_pEditor->playHeadX() - cx;
 	if (x >= rect.left() && x <= rect.right()) {
 		pPainter->setPen(Qt::red);
 		pPainter->drawLine(x, rect.top(), x, rect.bottom());
 	}
+
+	// Draw edit-head line...
+	x = m_pEditor->editHeadX() - cx;
+	if (x >= rect.left() && x <= rect.right()) {
+		pPainter->setPen(Qt::blue);
+		pPainter->drawLine(x, rect.top(), x, rect.bottom());
+	}
+
+	// Draw edit-tail line...
+	x = m_pEditor->editTailX() - cx;
+	if (x >= rect.left() && x <= rect.right()) {
+		pPainter->setPen(Qt::blue);
+		pPainter->drawLine(x, rect.top(), x, rect.bottom());
+	}
 }
 
 
@@ -541,15 +562,47 @@ void qtractorMidiEditEvent::keyPressEvent ( QKeyEvent *pKeyEvent )
 void qtractorMidiEditEvent::mousePressEvent ( QMouseEvent *pMouseEvent )
 {
 	// Process mouse press...
-	qtractorScrollView::mousePressEvent(pMouseEvent);
+//	qtractorScrollView::mousePressEvent(pMouseEvent);
 
-	// Only the left-mouse-button is meaningful atm...
-	if (pMouseEvent->button() != Qt::LeftButton)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
+	// Which mouse state?
+	const bool bModifier = (pMouseEvent->modifiers()
+		& (Qt::ShiftModifier | Qt::ControlModifier));
+
 	// Maybe start the drag-move-selection dance?
 	const QPoint& pos
 		= qtractorScrollView::viewportToContents(pMouseEvent->pos());
+	qtractorTimeScale *pTimeScale = m_pEditor->timeScale();
+	unsigned long iFrame = pTimeScale->frameSnap(m_pEditor->offset()
+		+ pTimeScale->frameFromPixel(pos.x() > 0 ? pos.x() : 0));
+
+	switch (pMouseEvent->button()) {
+	case Qt::LeftButton:
+		// Only the left-mouse-button was meaningful...
+		break;
+	case Qt::MidButton:
+		// Mid-button direct positioning...
+		m_pEditor->selectAll(this, false);
+		// Which mouse state?
+		if (bModifier) {
+			// Edit cursor (merge) positioning...
+			m_pEditor->setEditHead(iFrame);
+			m_pEditor->setEditTail(iFrame);
+		} else {
+			// Play-head positioning commit...
+			m_pEditor->setPlayHead(iFrame);
+			pSession->setPlayHead(m_pEditor->playHead());
+		}
+		// Logical contents changed, just for visual feedback...
+		m_pEditor->selectionChangeNotify();
+		// Fall thru...
+	default:
+		return;
+	}
+
 
 	// Remember what and where we'll be dragging/selecting...
 	m_pEditor->dragMoveStart(this, pos, pMouseEvent->modifiers());
@@ -560,7 +613,7 @@ void qtractorMidiEditEvent::mousePressEvent ( QMouseEvent *pMouseEvent )
 void qtractorMidiEditEvent::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 {
 	// Process mouse move...
-	qtractorScrollView::mouseMoveEvent(pMouseEvent);
+//	qtractorScrollView::mouseMoveEvent(pMouseEvent);
 
 	// Are we already moving/dragging something?
 	const QPoint& pos
