@@ -168,11 +168,11 @@ void qtractorMidiEditView::updateContentsWidth ( int iContentsWidth )
 
 	// Force an update on other views too...
 	m_pEditor->editTime()->resizeContents(
-		iContentsWidth + 100, m_pEditor->editTime()->contentsHeight());
+		iContentsWidth + 100, m_pEditor->editTime()->viewport()->height());
 //	m_pEditor->editTime()->updateContents();
 
 	m_pEditor->editEvent()->resizeContents(
-		iContentsWidth, m_pEditor->editEvent()->contentsHeight());
+		iContentsWidth, m_pEditor->editEvent()->viewport()->height());
 //	m_pEditor->editEvent()->updateContents();
 }
 
@@ -268,6 +268,10 @@ void qtractorMidiEditView::updatePixmap ( int cx, int cy )
 	m_pixmap = QPixmap(w, h);
 	m_pixmap.fill(rgbBase);
 
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
 	qtractorTimeScale *pTimeScale = m_pEditor->timeScale();
 	if (pTimeScale == NULL)
 		return;
@@ -320,27 +324,25 @@ void qtractorMidiEditView::updatePixmap ( int cx, int cy )
 	int x = pNode->pixelFromBeat(iBeat) - dx;
 	int x2 = x;
 	while (x < w) {
-		if (x >= 0) {
-			bool bBeatIsBar = pNode->beatIsBar(iBeat);
-			if (bBeatIsBar) {
-				p.setPen(rgbLight);
-				p.drawLine(x, 0, x, h);
-				if (m_pEditor->isSnapZebra() && (x > x2) && (++iBar & 1))
-					p.fillRect(QRect(x2, 0, x - x2 + 1, h), zebra);
-				x2 = x;
-				if (iBeat == pNode->beat)
-					iPixelsPerBeat = pNode->pixelsPerBeat();
-			}
-			if (bBeatIsBar || iPixelsPerBeat > 8) {
-				p.setPen(rgbDark);
-				p.drawLine(x - 1, 0, x - 1, h);
-			}
+		bool bBeatIsBar = pNode->beatIsBar(iBeat);
+		if (bBeatIsBar) {
+			p.setPen(rgbDark);
+			p.drawLine(x - 1, 0, x - 1, h);
+			if (m_pEditor->isSnapZebra() && (x > x2) && (++iBar & 1))
+				p.fillRect(QRect(x2, 0, x - x2 + 1, h), zebra);
+			x2 = x;
+			if (iBeat == pNode->beat)
+				iPixelsPerBeat = pNode->pixelsPerBeat();
+		}
+		if (bBeatIsBar || iPixelsPerBeat > 8) {
+			p.setPen(rgbLight);
+			p.drawLine(x, 0, x, h);
 		}
 		if (iSnapPerBeat > 1) {
 			int q = iPixelsPerBeat / iSnapPerBeat;
 			if (q > 4) {  
 				p.setPen(rgbBase.value() < 0x7f
-					? rgbDark.darker(105) : rgbLight.lighter(120));
+					? rgbLight.darker(105) : rgbLight.lighter(120));
 				for (int i = 1; i < iSnapPerBeat; ++i) {
 					x = pTimeScale->pixelSnap(x + dx + q) - dx - 1;
 					p.drawLine(x, 0, x, h);
@@ -356,6 +358,17 @@ void qtractorMidiEditView::updatePixmap ( int cx, int cy )
 	if (y > ch)
 		p.fillRect(0, ch, w, h - ch, rgbDark);
 
+	// Draw location marker lines...
+	qtractorTimeScale::Marker *pMarker
+		= pTimeScale->markers().seekPixel(dx);
+	while (pMarker) {
+		x = pTimeScale->pixelFromFrame(pMarker->frame) - dx;
+		if (x > w) break;
+		p.setPen(pMarker->color);
+		p.drawLine(x, 0, x, h);
+		pMarker = pMarker->next();
+	}
+
 	//
 	// Draw the sequence events...
 	//
@@ -409,10 +422,6 @@ void qtractorMidiEditView::updatePixmap ( int cx, int cy )
 		pEvent = pEvent->next();
 	}
 
-	qtractorSession *pSession = qtractorSession::getInstance();
-	if (pSession == NULL)
-		return;
-
 	// Draw loop boundaries, if applicable...
 	if (pSession->isLooping()) {
 		const QBrush shade(QColor(0, 0, 0, 60));
@@ -467,13 +476,27 @@ void qtractorMidiEditView::drawContents ( QPainter *pPainter, const QRect& rect
 
 	m_pEditor->paintDragState(this, pPainter);
 
-	// Draw special play-head header...
+	// Draw special play-head line...
 	int cx = qtractorScrollView::contentsX();
 	int x  = m_pEditor->playHeadX() - cx;
 	if (x >= rect.left() && x <= rect.right()) {
 		pPainter->setPen(Qt::red);
 		pPainter->drawLine(x, rect.top(), x, rect.bottom());
 	}
+
+	// Draw edit-head line...
+	x = m_pEditor->editHeadX() - cx;
+	if (x >= rect.left() && x <= rect.right()) {
+		pPainter->setPen(Qt::blue);
+		pPainter->drawLine(x, rect.top(), x, rect.bottom());
+	}
+
+	// Draw edit-tail line...
+	x = m_pEditor->editTailX() - cx;
+	if (x >= rect.left() && x <= rect.right()) {
+		pPainter->setPen(Qt::blue);
+		pPainter->drawLine(x, rect.top(), x, rect.bottom());
+	}
 }
 
 
@@ -514,15 +537,46 @@ void qtractorMidiEditView::keyPressEvent ( QKeyEvent *pKeyEvent )
 void qtractorMidiEditView::mousePressEvent ( QMouseEvent *pMouseEvent )
 {
 	// Process mouse press...
-	qtractorScrollView::mousePressEvent(pMouseEvent);
+//	qtractorScrollView::mousePressEvent(pMouseEvent);
 
-	// Only the left-mouse-button is meaningful atm...
-	if (pMouseEvent->button() != Qt::LeftButton)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
+	// Which mouse state?
+	const bool bModifier = (pMouseEvent->modifiers()
+		& (Qt::ShiftModifier | Qt::ControlModifier));
+
 	// Maybe start the drag-move-selection dance?
 	const QPoint& pos
 		= qtractorScrollView::viewportToContents(pMouseEvent->pos());
+	qtractorTimeScale *pTimeScale = m_pEditor->timeScale();
+	unsigned long iFrame = pTimeScale->frameSnap(m_pEditor->offset()
+		+ pTimeScale->frameFromPixel(pos.x() > 0 ? pos.x() : 0));
+
+	switch (pMouseEvent->button()) {
+	case Qt::LeftButton:
+		// Only the left-mouse-button was meaningful...
+		break;
+	case Qt::MidButton:
+		// Mid-button direct positioning...
+		m_pEditor->selectAll(this, false);
+		// Which mouse state?
+		if (bModifier) {
+			// Edit cursor (merge) positioning...
+			m_pEditor->setEditHead(iFrame);
+			m_pEditor->setEditTail(iFrame);
+		} else {
+			// Play-head positioning commit...
+			m_pEditor->setPlayHead(iFrame);
+			pSession->setPlayHead(m_pEditor->playHead());
+		}
+		// Logical contents changed, just for visual feedback...
+		m_pEditor->selectionChangeNotify();
+		// Fall thru...
+	default:
+		return;
+	}
 
 	// Remember what and where we'll be dragging/selecting...
 	m_pEditor->dragMoveStart(this, pos, pMouseEvent->modifiers());
@@ -533,7 +587,7 @@ void qtractorMidiEditView::mousePressEvent ( QMouseEvent *pMouseEvent )
 void qtractorMidiEditView::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 {
 	// Process mouse move...
-	qtractorScrollView::mouseMoveEvent(pMouseEvent);
+//	qtractorScrollView::mouseMoveEvent(pMouseEvent);
 
 	// Are we already moving/dragging something?
 	const QPoint& pos
@@ -547,7 +601,7 @@ void qtractorMidiEditView::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 void qtractorMidiEditView::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 {
 	// Process mouse release...
-	qtractorScrollView::mouseReleaseEvent(pMouseEvent);
+//	qtractorScrollView::mouseReleaseEvent(pMouseEvent);
 
 	// Were we moving/dragging something?
 	const QPoint& pos
