@@ -104,8 +104,28 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	m_pSnapToScaleTypeComboBox = new QComboBox(m_ui.snapToScaleToolbar);
 	m_pSnapToScaleKeyComboBox->setEditable(false);
 
+	// View/Snap-to-beat actions initialization...
+	int iSnap = 0;
+	const QString sSnapObjectName("viewSnapPerBeat%1");
+	const QString sSnapStatusTip(tr("Set current snap to %1"));
+	const QStringList& snapItems = qtractorTimeScale::snapItems();
+	QStringListIterator snapIter(snapItems);
+	while (snapIter.hasNext()) {
+		const QString& sSnapText = snapIter.next();
+		QAction *pAction = new QAction(sSnapText, this);
+		pAction->setObjectName(sSnapObjectName.arg(iSnap));
+		pAction->setStatusTip(sSnapStatusTip.arg(sSnapText));
+		pAction->setCheckable(true);
+		pAction->setData(iSnap++);
+		QObject::connect(pAction,
+			SIGNAL(triggered(bool)),
+			SLOT(viewSnap()));
+		m_snapPerBeatActions.append(pAction);
+		addAction(pAction);
+	}
+
 	// Pre-fill the combo-boxes...
-	m_pSnapPerBeatComboBox->insertItems(0, qtractorTimeScale::snapItems());
+	m_pSnapPerBeatComboBox->insertItems(0, snapItems);
 
 	const QIcon icon(":/images/itemProperty.png");
 
@@ -295,6 +315,9 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	QObject::connect(m_ui.editSelectRangeAction,
 		SIGNAL(triggered(bool)),
 		SLOT(editSelectRange()));
+	QObject::connect(m_ui.editInsertRangeAction,
+		SIGNAL(triggered(bool)),
+		SLOT(editInsertRange()));
 
 	QObject::connect(m_ui.toolsQuantizeAction,
 		SIGNAL(triggered(bool)),
@@ -557,6 +580,10 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 // Destructor.
 qtractorMidiEditorForm::~qtractorMidiEditorForm (void)
 {
+	// View/Snap-to-beat actions termination...
+	qDeleteAll(m_snapPerBeatActions);
+	m_snapPerBeatActions.clear();
+
 	// Drop any widgets around (not really necessary)...
 	if (m_pMidiEventList)
 		delete m_pMidiEventList;
@@ -1174,28 +1201,51 @@ void qtractorMidiEditorForm::editModeDraw ( bool bOn )
 // Select none contents.
 void qtractorMidiEditorForm::editSelectNone (void)
 {
-	m_pMidiEditor->selectAll(false, false);
+	qtractorScrollView *pScrollView = m_pMidiEditor->editView();
+	if (m_pMidiEditor->editEvent()->hasFocus())
+		pScrollView = m_pMidiEditor->editEvent();
+
+	m_pMidiEditor->selectAll(pScrollView, false, false);
 }
 
 
 // Select invert contents.
 void qtractorMidiEditorForm::editSelectInvert (void)
 {
-	m_pMidiEditor->selectAll(true, true);
+	qtractorScrollView *pScrollView = m_pMidiEditor->editView();
+	if (m_pMidiEditor->editEvent()->hasFocus())
+		pScrollView = m_pMidiEditor->editEvent();
+
+	m_pMidiEditor->selectAll(pScrollView, true, true);
 }
 
 
 // Select all contents.
 void qtractorMidiEditorForm::editSelectAll (void)
 {
-	m_pMidiEditor->selectAll(true, false);
+	qtractorScrollView *pScrollView = m_pMidiEditor->editView();
+	if (m_pMidiEditor->editEvent()->hasFocus())
+		pScrollView = m_pMidiEditor->editEvent();
+
+	m_pMidiEditor->selectAll(pScrollView, true, false);
 }
 
 
 // Select contents range.
 void qtractorMidiEditorForm::editSelectRange (void)
 {
-	m_pMidiEditor->selectRange(true, true);
+	qtractorScrollView *pScrollView = m_pMidiEditor->editView();
+	if (m_pMidiEditor->editEvent()->hasFocus())
+		pScrollView = m_pMidiEditor->editEvent();
+
+	m_pMidiEditor->selectRange(pScrollView, true, true);
+}
+
+
+// Insert range.
+void qtractorMidiEditorForm::editInsertRange (void)
+{
+	m_pMidiEditor->insertEditRange();
 }
 
 
@@ -1602,6 +1652,7 @@ void qtractorMidiEditorForm::stabilizeForm (void)
 	m_ui.editDeleteAction->setEnabled(bSelected);
 	m_ui.editModeDrawAction->setEnabled(m_pMidiEditor->isEditMode());
 	m_ui.editSelectNoneAction->setEnabled(bSelected);
+	m_ui.editInsertRangeAction->setEnabled(m_pMidiEditor->isInsertable());
 
 #if 0
 	m_ui.toolsMenu->setEnabled(bSelected);
@@ -1747,13 +1798,12 @@ void qtractorMidiEditorForm::updateSnapMenu (void)
 		= qtractorTimeScale::indexFromSnap(pTimeScale->snapPerBeat());
 
 	int iSnap = 0;
-	QStringListIterator iter(qtractorTimeScale::snapItems());
+	QListIterator<QAction *> iter(m_snapPerBeatActions);
 	while (iter.hasNext()) {
-		QAction *pAction = m_ui.viewSnapMenu->addAction(
-			iter.next(), this, SLOT(viewSnap()));
-		pAction->setCheckable(true);
+		QAction *pAction = iter.next();
 		pAction->setChecked(iSnap == iSnapCurrent);
-		pAction->setData(iSnap++);
+		m_ui.viewSnapMenu->addAction(pAction);
+		++iSnap;
 	}
 
 	m_ui.viewSnapMenu->addSeparator();
