@@ -119,6 +119,7 @@ protected slots:
 	void editSelectNone();
 	void editSelectInvert();
 	void editSelectRange();
+	void editInsertRange();
 
 	void toolsQuantize();
 	void toolsTranspose();
@@ -206,6 +207,9 @@ private:
 	// Edit-mode action group up.
 	QActionGroup *m_pEditModeActionGroup;
 
+	// View/Snap-to-beat actions (for shortcuts access)
+	QList<QAction *> m_snapPerBeatActions;
+
 	// Edit snap mode.
 	QComboBox *m_pSnapPerBeatComboBox;
 
