@@ -91,6 +91,9 @@ public:
 	// Constructor.
 	qtractorMidiEngine(qtractorSession *pSession);
 
+	// Engine initialization.
+	bool init();
+
 	// Special event notifier proxy object.
 	const qtractorMidiEngineProxy *proxy() const;
 
@@ -140,10 +143,8 @@ public:
 	void restartLoop();
 
 	// The delta-time/frame accessors.
-	void setTimeStart(long iTimeStart);
 	long timeStart() const;
 
-	void setFrameStart(long iFrameStart);
 	long frameStart() const;
 
 	// Special track-immediate methods.
@@ -197,7 +198,6 @@ public:
 	// Audition/pre-listening bus mode accessors.
 	void setPlayerBus(bool bPlayerBus);
 	bool isPlayerBus() const;
-	void resetPlayerBus();
 
 	bool isPlayerOpen() const;
 	bool openPlayer(const QString& sFilename, int iTrackChannel = -1);
@@ -256,7 +256,6 @@ public:
 protected:
 
 	// Concrete device (de)activation methods.
-	bool init();
 	bool activate();
 	bool start();
 	void stop();
