@@ -1,7 +1,7 @@
 // qtractorMidiEvent.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -74,19 +74,20 @@ public:
 		unsigned char data1 = 0, unsigned char data2 = 0,
 		unsigned long duration = 0)
 		: m_time(time), m_type(type)
-		{ m_data[0] = data1; m_data[1] = data2; m_u.duration = duration; }
+		{ m_v.data[0] = data1; m_v.data[1] = data2; m_u.duration = duration; }
 
 	// Copy constructor.
 	qtractorMidiEvent(const qtractorMidiEvent& e)
 		: qtractorList<qtractorMidiEvent>::Link(),
 		m_time(e.m_time), m_type(e.m_type)
 	{
-		m_data[0] = e.m_data[0]; m_data[1] = e.m_data[1]; 
 		if (m_type == SYSEX) {
-			unsigned short iSysex = e.sysex_len();
-			m_u.pSysex = new unsigned char [iSysex];
-			::memcpy(m_u.pSysex, e.sysex(), iSysex);
+			m_v.iSysex = e.m_v.iSysex;
+			m_u.pSysex = new unsigned char [m_v.iSysex];
+			::memcpy(m_u.pSysex, e.m_u.pSysex, m_v.iSysex);
 		} else {
+			m_v.data[0] = e.m_v.data[0];
+			m_v.data[1] = e.m_v.data[1]; 
 			m_u.duration = e.m_u.duration;
 		}
 	}
@@ -100,10 +101,10 @@ public:
 	EventType     type()       const { return m_type; }
 
 	// Underloaded accessors (getters).
-	unsigned char note()       const { return m_data[0]; }
-	unsigned char velocity()   const { return m_data[1]; }
-	unsigned char controller() const { return m_data[0]; }
-	unsigned char value()      const { return m_data[1]; }
+	unsigned char note()       const { return m_v.data[0]; }
+	unsigned char velocity()   const { return m_v.data[1]; }
+	unsigned char controller() const { return m_v.data[0]; }
+	unsigned char value()      const { return m_v.data[1]; }
 
 	// Event properties accessors (setters).
 	void setTime(unsigned long time) { m_time = time; }
@@ -114,10 +115,10 @@ public:
 		{ m_time = (m_time > iOffset ? m_time - iOffset : 0); }
 
 	// Underloaded accessors (setters).
-	void setNote(unsigned char note)             { m_data[0] = note; }
-	void setVelocity(unsigned char velocity)     { m_data[1] = velocity; }
-	void setController(unsigned char controller) { m_data[0] = controller; }
-	void setValue(unsigned char value)           { m_data[1] = value; }
+	void setNote(unsigned char note)             { m_v.data[0] = note; }
+	void setVelocity(unsigned char velocity)     { m_v.data[1] = velocity; }
+	void setController(unsigned char controller) { m_v.data[0] = controller; }
+	void setValue(unsigned char value)           { m_v.data[1] = value; }
 
 	// Duration accessors (NOTEON).
 	void setDuration(unsigned long duration) { m_u.duration = duration; }
@@ -125,29 +126,29 @@ public:
 
 	// Sysex data accessors (SYSEX).
 	unsigned char *sysex()     const { return m_u.pSysex; }
-	unsigned short sysex_len() const { return *(unsigned short *)(&m_data[0]); }
+	unsigned short sysex_len() const { return m_v.iSysex; }
 
 	// Allocate and set a new sysex buffer.
 	void setSysex(unsigned char *pSysex, unsigned short iSysex)
 	{
 		if (m_type == SYSEX && m_u.pSysex) delete [] m_u.pSysex;
-		*(unsigned short *)(&m_data[0]) = iSysex;
-		m_u.pSysex = new unsigned char [iSysex];
-		::memcpy(m_u.pSysex, pSysex, iSysex);
+		m_v.iSysex = iSysex;
+		m_u.pSysex = new unsigned char [m_v.iSysex];
+		::memcpy(m_u.pSysex, pSysex, m_v.iSysex);
 	}
 
 	// Special accessors for pitch-bend event types.
 	int pitchBend() const
 	{
-		unsigned short val = ((unsigned short) m_data[1] << 7) | m_data[0];
+		unsigned short val = ((unsigned short) m_v.data[1] << 7) | m_v.data[0];
 		return int(val) - 0x2000;
 	}
 	
 	void setPitchBend(int iPitchBend)
 	{
 		unsigned short val = (unsigned short) (0x2000 + iPitchBend);
-		m_data[0] = (val & 0x007f);
-		m_data[1] = (val & 0x3f80) >> 7;
+		m_v.data[0] = (val & 0x007f);
+		m_v.data[1] = (val & 0x3f80) >> 7;
 	}
 
 private:
@@ -155,7 +156,13 @@ private:
 	// Event instance members.
 	unsigned long  m_time;
 	EventType      m_type;
-	unsigned char  m_data[2];
+
+	// Nominal event data.
+	union {
+		unsigned char  data[2];		// type != SYSEX
+		unsigned short iSysex;		// type == SYSEX
+	} m_v;
+
 	// Extra event data.
 	union {
 		unsigned long  duration;	// type == NOTEON
