@@ -1,7 +1,7 @@
 // qtractorMidiFile.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2012, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -96,7 +96,12 @@ protected:
 	int writeData (unsigned char *pData, unsigned short n);
 
 	// Write tempo-time-signature node.
-	void writeNode(qtractorMidiFileTempo::Node *pNode, unsigned long iLastTime);
+	void writeNode(
+		qtractorMidiFileTempo::Node *pNode, unsigned long iLastTime);
+
+	// Write location marker.
+	void writeMarker(
+		qtractorMidiFileTempo::Marker *pMarker, unsigned long iLastTime);
 
 private:
 
