@@ -51,6 +51,9 @@ public:
 		qtractorObserverWidget<Widget> *widget() const
 			{ return m_pWidget; }
 
+		// Virtual destructor.
+		virtual ~Interface() {}
+
 		// Pure virtuals.
 		virtual float scaleFromValue(float fValue) const = 0;
 		virtual float valueFromScale(float fScale) const = 0;
