@@ -1,7 +1,7 @@
 // qtractorOptions.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2012, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -181,10 +181,15 @@ public:
 	QStringList vstPaths;
 	QStringList lv2Paths;
 
+	QString sLv2PresetDir;
+
 	// Plug-in instrument options.
 	bool bAudioOutputBus;
 	bool bAudioOutputAutoConnect;
 
+	// Plug-in GUI options.
+	bool bOpenEditor;
+
 	// VST dummy plugin scan option.
 	bool bDummyVstScan;
 
@@ -201,6 +206,7 @@ public:
 	// Tracks view options...
 	int  iTrackViewSelectMode;
 	bool bTrackViewDropSpan;
+	bool bTrackViewSnapZebra;
 	bool bTrackViewSnapGrid;
 	bool bTrackViewToolTips;
 	bool bTrackViewCurveEdit;
@@ -224,6 +230,7 @@ public:
 	int  iMidiHorizontalZoom;
 	int  iMidiVerticalZoom;
 	int  iMidiSnapPerBeat;
+	bool bMidiSnapZebra;
 	bool bMidiSnapGrid;
 	bool bMidiToolTips;
 	int  iMidiSnapToScaleKey;
