@@ -1,7 +1,7 @@
 // qtractorShortcutForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2012, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@
 
 *****************************************************************************/
 
+#include "qtractorAbout.h"
 #include "qtractorShortcutForm.h"
 
 #include "qtractorOptions.h"
@@ -49,49 +50,44 @@ public:
 		: QTableWidgetItem(sText)
 		{ setFlags(flags() & ~Qt::ItemIsEditable); }
 	qtractorShortcutTableItem(const QKeySequence& shortcut)
-		: QTableWidgetItem(QString(shortcut)) {}
+		: QTableWidgetItem(shortcut.toString()) {}
 };
 
 
 //-------------------------------------------------------------------------
 // qtractorShortcutTableItemEdit
 
-class qtractorShortcutTableItemEdit : public QLineEdit
+// Shortcut key to text event translation.
+void qtractorShortcutTableItemEdit::keyPressEvent ( QKeyEvent *pKeyEvent )
 {
-public:
+	int iKey = pKeyEvent->key();
+	Qt::KeyboardModifiers modifiers = pKeyEvent->modifiers();
 
-	// Constructor.
-	qtractorShortcutTableItemEdit(QWidget *pParent = NULL)
-		: QLineEdit(pParent) {}
-
-protected:
-
-	// Shortcut key to text event translation.
-	void keyPressEvent(QKeyEvent *pKeyEvent)
-	{
-		int iKey = pKeyEvent->key();
-		Qt::KeyboardModifiers modifiers = pKeyEvent->modifiers();
-
-		if (iKey == Qt::Key_Return && modifiers == Qt::NoModifier) {
+	if (modifiers == Qt::NoModifier) {
+		switch (iKey) {
+		case Qt::Key_Return:
 			emit editingFinished();
 			return;
-		}
-	
-		if (iKey >= Qt::Key_Shift && iKey < Qt::Key_F1) {
-			QLineEdit::keyPressEvent(pKeyEvent);
+		case Qt::Key_Escape:
+			emit editingCanceled();
 			return;
 		}
-
-		if (modifiers & Qt::ShiftModifier)
-			iKey |= Qt::SHIFT;
-		if (modifiers & Qt::ControlModifier)
-			iKey |= Qt::CTRL;
-		if (modifiers & Qt::AltModifier)
-			iKey |= Qt::ALT;
-
-		QLineEdit::setText(QString(QKeySequence(iKey)));
 	}
-};
+
+	if (iKey >= Qt::Key_Shift && iKey < Qt::Key_F1) {
+		QLineEdit::keyPressEvent(pKeyEvent);
+		return;
+	}
+
+	if (modifiers & Qt::ShiftModifier)
+		iKey |= Qt::SHIFT;
+	if (modifiers & Qt::ControlModifier)
+		iKey |= Qt::CTRL;
+	if (modifiers & Qt::AltModifier)
+		iKey |= Qt::ALT;
+
+	QLineEdit::setText(QString(QKeySequence(iKey)));
+}
 
 
 //-------------------------------------------------------------------------
@@ -100,7 +96,7 @@ protected:
 qtractorShortcutTableItemEditor::qtractorShortcutTableItemEditor (
 	QWidget *pParent ) : QWidget(pParent)
 {
-	m_pLineEdit = new qtractorShortcutTableItemEdit(/*this*/);
+	m_pItemEdit = new qtractorShortcutTableItemEdit(/*this*/);
 
 	m_pToolButton = new QToolButton(/*this*/);
 	m_pToolButton->setFixedWidth(18);
@@ -109,16 +105,19 @@ qtractorShortcutTableItemEditor::qtractorShortcutTableItemEditor (
 	QHBoxLayout *pLayout = new QHBoxLayout();
 	pLayout->setSpacing(0);
 	pLayout->setMargin(0);
-	pLayout->addWidget(m_pLineEdit);
+	pLayout->addWidget(m_pItemEdit);
 	pLayout->addWidget(m_pToolButton);
 	QWidget::setLayout(pLayout);
 	
 	QWidget::setFocusPolicy(Qt::StrongFocus);
-	QWidget::setFocusProxy(m_pLineEdit);
+	QWidget::setFocusProxy(m_pItemEdit);
 
-	QObject::connect(m_pLineEdit,
+	QObject::connect(m_pItemEdit,
 		SIGNAL(editingFinished()),
 		SLOT(finish()));
+	QObject::connect(m_pItemEdit,
+		SIGNAL(editingCanceled()),
+		SLOT(cancel()));
 	QObject::connect(m_pToolButton,
 		SIGNAL(clicked()),
 		SLOT(clear()));
@@ -128,32 +127,47 @@ qtractorShortcutTableItemEditor::qtractorShortcutTableItemEditor (
 // Shortcut text accessors.
 void qtractorShortcutTableItemEditor::setText ( const QString& sText )
 {
-	m_pLineEdit->setText(sText);
+	m_pItemEdit->setText(sText);
 }
 
 
 QString qtractorShortcutTableItemEditor::text (void) const
 {
-	return m_pLineEdit->text();
+	return m_pItemEdit->text();
 }
 
 
 // Shortcut text clear/toggler.
 void qtractorShortcutTableItemEditor::clear (void)
 {
-	if (m_pLineEdit->text() == m_sDefaultText)
-		m_pLineEdit->clear();
+	if (m_pItemEdit->text() == m_sDefaultText)
+		m_pItemEdit->clear();
 	else
-		m_pLineEdit->setText(m_sDefaultText);
+		m_pItemEdit->setText(m_sDefaultText);
 
-	m_pLineEdit->setFocus();
+	m_pItemEdit->setFocus();
 }
 
 
 // Shortcut text finish notification.
 void qtractorShortcutTableItemEditor::finish (void)
 {
+	bool bBlockSignals = m_pItemEdit->blockSignals(true);
 	emit editingFinished();
+	m_index = QModelIndex();
+	m_sDefaultText.clear();
+	m_pItemEdit->blockSignals(bBlockSignals);
+}
+
+
+// Shortcut text cancel notification.
+void qtractorShortcutTableItemEditor::cancel (void)
+{
+	bool bBlockSignals = m_pItemEdit->blockSignals(true);
+	m_index = QModelIndex();
+	m_sDefaultText.clear();
+	emit editingFinished();
+	m_pItemEdit->blockSignals(bBlockSignals);
 }
 
 
@@ -161,8 +175,8 @@ void qtractorShortcutTableItemEditor::finish (void)
 // qtractorShortcutTableItemDelegate
 
 qtractorShortcutTableItemDelegate::qtractorShortcutTableItemDelegate (
-	QTableWidget *pTableWidget )
-	: QItemDelegate(pTableWidget), m_pTableWidget(pTableWidget)
+	qtractorShortcutForm *pShortcutForm )
+	: QItemDelegate(pShortcutForm->tableWidget()), m_pShortcutForm(pShortcutForm)
 {
 }
 
@@ -173,7 +187,8 @@ void qtractorShortcutTableItemDelegate::paint ( QPainter *pPainter,
 {
 	// Special treatment for action icon+text...
 	if (index.column() == 0) {
-		QTableWidgetItem *pItem = m_pTableWidget->item(index.row(), 0);
+		QTableWidget *pTableWidget = m_pShortcutForm->tableWidget();
+		QTableWidgetItem *pItem	= pTableWidget->item(index.row(), 0);
 		pPainter->save();
 		if (option.state & QStyle::State_Selected) {
 			const QPalette& pal = option.palette;
@@ -182,7 +197,7 @@ void qtractorShortcutTableItemDelegate::paint ( QPainter *pPainter,
 		}
 		// Draw the icon...
 		QRect rect = option.rect;
-		const QSize& iconSize = m_pTableWidget->iconSize();
+		const QSize& iconSize = pTableWidget->iconSize();
 		pPainter->drawPixmap(1,
 			rect.top() + ((rect.height() - iconSize.height()) >> 1),
 			pItem->icon().pixmap(iconSize));
@@ -204,6 +219,7 @@ QWidget *qtractorShortcutTableItemDelegate::createEditor ( QWidget *pParent,
 {
 	qtractorShortcutTableItemEditor *pItemEditor
 		= new qtractorShortcutTableItemEditor(pParent);
+	pItemEditor->setIndex(index);
 	pItemEditor->setDefaultText(
 		index.model()->data(index, Qt::DisplayRole).toString());
 	QObject::connect(pItemEditor,
@@ -236,7 +252,10 @@ void qtractorShortcutTableItemDelegate::commitEditor (void)
 {
 	qtractorShortcutTableItemEditor *pItemEditor
 		= qobject_cast<qtractorShortcutTableItemEditor *> (sender());
-	emit commitData(pItemEditor);
+
+	if (m_pShortcutForm->commitEditor(pItemEditor))
+		emit commitData(pItemEditor);
+
 	emit closeEditor(pItemEditor);
 }
 
@@ -244,8 +263,8 @@ void qtractorShortcutTableItemDelegate::commitEditor (void)
 //-------------------------------------------------------------------------
 // qtractorShortcutForm
 
-qtractorShortcutForm::qtractorShortcutForm ( QList<QAction *> actions,
-	QWidget *pParent ) : QDialog(pParent)
+qtractorShortcutForm::qtractorShortcutForm (
+	const QList<QAction *>& actions, QWidget *pParent ) : QDialog(pParent)
 {
 	// Setup UI struct...
 	m_ui.setupUi(this);
@@ -258,7 +277,7 @@ qtractorShortcutForm::qtractorShortcutForm ( QList<QAction *> actions,
 //	m_ui.qtractorShortcutTable = new QTableWidget(0, 3, this);
 	m_ui.ShortcutTable->setIconSize(QSize(16, 16));
 	m_ui.ShortcutTable->setItemDelegate(
-		new qtractorShortcutTableItemDelegate(m_ui.ShortcutTable));
+		new qtractorShortcutTableItemDelegate(this));
 //	m_ui.ShortcutTable->setSelectionMode(QAbstractItemView::SingleSelection);
 //	m_ui.ShortcutTable->setSelectionBehavior(QAbstractItemView::SelectRows);
 //	m_ui.ShortcutTable->setAlternatingRowColors(true);
@@ -286,9 +305,13 @@ qtractorShortcutForm::qtractorShortcutForm ( QList<QAction *> actions,
 			new qtractorShortcutTableItem(pAction->icon(), pAction->text()));
 		m_ui.ShortcutTable->setItem(iRow, 1,
 			new qtractorShortcutTableItem(pAction->statusTip()));
+		const QKeySequence& shortcut = pAction->shortcut();
+		const QString& sShortcutText = shortcut.toString();
 		m_ui.ShortcutTable->setItem(iRow, 2,
-			new qtractorShortcutTableItem(pAction->shortcut()));
-		m_actions.append(pAction);
+			new qtractorShortcutTableItem(shortcut));
+		m_actions.insert(pAction, iRow);
+		if (!sShortcutText.isEmpty())
+			m_shortcuts.insert(sShortcutText, iRow);
 		++iRow;
 	}
 
@@ -323,6 +346,46 @@ qtractorShortcutForm::~qtractorShortcutForm (void)
 }
 
 
+QTableWidget *qtractorShortcutForm::tableWidget (void) const
+{
+	return m_ui.ShortcutTable;
+}
+
+
+// Shortcut action finder & settler.
+bool qtractorShortcutForm::commitEditor (
+	qtractorShortcutTableItemEditor *pItemEditor )
+{
+	const QModelIndex& index = pItemEditor->index();
+	if (!index.isValid())
+		return false;
+
+	const QString& sShortcutText = pItemEditor->text();
+	const QString& sDefaultText = pItemEditor->defaultText();
+
+	if (sShortcutText == sDefaultText)
+		return false;
+
+	if (!sShortcutText.isEmpty()) {
+		if (m_shortcuts.contains(sShortcutText)) {
+			QMessageBox::warning(this,
+				tr("Warning") + " - " QTRACTOR_TITLE,
+				tr("Keyboard shortcut (%1) already assigned.")
+					.arg(sShortcutText),
+				QMessageBox::Cancel);
+			pItemEditor->clear();
+			return false;
+		}
+		m_shortcuts.insert(sShortcutText, index.row());
+	}
+
+	if (!sDefaultText.isEmpty())
+		m_shortcuts.remove(sDefaultText);
+
+	return true;
+}
+
+
 void qtractorShortcutForm::actionActivated ( QTableWidgetItem *pItem )
 {
 	m_ui.ShortcutTable->editItem(m_ui.ShortcutTable->item(pItem->row(), 2));
@@ -331,7 +394,9 @@ void qtractorShortcutForm::actionActivated ( QTableWidgetItem *pItem )
 
 void qtractorShortcutForm::actionChanged ( QTableWidgetItem *pItem )
 {
-	pItem->setText(QString(QKeySequence(pItem->text().trimmed())));
+	const QString& sShortcutText
+		= QKeySequence(pItem->text().trimmed()).toString();
+	pItem->setText(sShortcutText);
 	++m_iDirtyCount;
 }
 
@@ -339,10 +404,11 @@ void qtractorShortcutForm::actionChanged ( QTableWidgetItem *pItem )
 void qtractorShortcutForm::accept (void)
 {
 	if (m_iDirtyCount > 0) {
-		for (int iRow = 0; iRow < m_actions.count(); ++iRow) {
-			QAction *pAction = m_actions[iRow];
-			pAction->setShortcut(
-				QKeySequence(m_ui.ShortcutTable->item(iRow, 2)->text()));
+		QHash<QAction *, int>::ConstIterator iter = m_actions.constBegin();
+		for ( ; iter != m_actions.constEnd(); ++iter) {
+			const QString& sShortcutText
+				= m_ui.ShortcutTable->item(iter.value(), 2)->text();
+			iter.key()->setShortcut(QKeySequence(sShortcutText));
 		}
 	}
 
@@ -361,7 +427,7 @@ void qtractorShortcutForm::reject (void)
 		if (m_ui.DialogButtonBox->button(QDialogButtonBox::Ok)->isEnabled())
 			buttons |= QMessageBox::Apply;
 		switch (QMessageBox::warning(this,
-			tr("Warning"), // + " - " QTRACTOR_TITLE,
+			tr("Warning") + " - " QTRACTOR_TITLE,
 			tr("Keyboard shortcuts have been changed.\n\n"
 			"Do you want to apply the changes?"),
 			buttons)) {
