@@ -1,7 +1,7 @@
 // qtractorShortcutForm.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2012, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -25,7 +25,7 @@
 #include "ui_qtractorShortcutForm.h"
 
 
-#include <QList>
+#include <QHash>
 #include <QItemDelegate>
 #include <QLineEdit>
 
@@ -33,7 +33,30 @@
 class QAction;
 class QToolButton;
 
-class qtractorShortcutTableItemEdit;
+
+//-------------------------------------------------------------------------
+// qtractorShortcutTableItemEdit
+
+class qtractorShortcutTableItemEdit : public QLineEdit
+{
+	Q_OBJECT
+
+public:
+
+	// Constructor.
+	qtractorShortcutTableItemEdit(QWidget *pParent = NULL)
+		: QLineEdit(pParent) {}
+
+signals:
+
+	// Custom cancel signal.
+	void editingCanceled();
+
+protected:
+
+	// Shortcut key to text event translation.
+	void keyPressEvent(QKeyEvent *pKeyEvent);
+};
 
 
 //-------------------------------------------------------------------------
@@ -52,6 +75,12 @@ public:
 	void setText(const QString& sText);
 	QString text() const;
 
+	// Index model row accessors.
+	void setIndex(const QModelIndex& index)
+		{ m_index = index; }
+	const QModelIndex& index() const
+		{ return m_index; }
+
 	// Default (initial) shortcut text accessors.
 	void setDefaultText(const QString& sDefaultText)
 		{ m_sDefaultText = sDefaultText; }
@@ -61,23 +90,30 @@ public:
 signals:
 
 	void editingFinished();
+	void editingCanceled();
+
+public slots:
+
+	void clear();
 
 protected slots:
 
-	void clear();
 	void finish();
+	void cancel();
 
 private:
 
 	// Instance variables.
-	qtractorShortcutTableItemEdit *m_pLineEdit;
+	qtractorShortcutTableItemEdit *m_pItemEdit;
 	QToolButton *m_pToolButton;
 	QString m_sDefaultText;
+	QModelIndex m_index;
 };
 
 
 //-------------------------------------------------------------------------
 // qtractorShortcutTableItemDelegate
+class qtractorShortcutForm;
 
 class qtractorShortcutTableItemDelegate : public QItemDelegate
 {
@@ -86,7 +122,7 @@ class qtractorShortcutTableItemDelegate : public QItemDelegate
 public:
 
 	// Constructor.
-	qtractorShortcutTableItemDelegate(QTableWidget *pTableWidget);
+	qtractorShortcutTableItemDelegate(qtractorShortcutForm *pShortcutForm);
 
 protected:
 
@@ -110,7 +146,7 @@ protected slots:
 
 private:
 
-	QTableWidget *m_pTableWidget;
+	qtractorShortcutForm *m_pShortcutForm;
 };
 
 
@@ -124,12 +160,17 @@ class qtractorShortcutForm : public QDialog
 public:
 
 	// Constructor.
-	qtractorShortcutForm(QList<QAction *> actions, QWidget *pParent = NULL);
+	qtractorShortcutForm(const QList<QAction *>& actions, QWidget *pParent = NULL);
 	
 	// Destructor.
-	
 	~qtractorShortcutForm();
 
+	// Shortcut table widget accessor.
+	QTableWidget *tableWidget() const;
+
+	// Shortcut action finder & settler.
+	bool commitEditor(qtractorShortcutTableItemEditor *pItemEditor);
+
 protected slots:
 
 	void actionActivated(QTableWidgetItem *);
@@ -143,7 +184,8 @@ private:
 	// The Qt-designer UI struct...
 	Ui::qtractorShortcutForm m_ui;
 
-	QList<QAction *> m_actions;
+	QHash<QAction *, int> m_actions;
+	QHash<QString, int> m_shortcuts;
 
 	int m_iDirtyCount;
 };
