@@ -30,6 +30,11 @@
 void qtractorTimeScale::reset (void)
 {
 	m_nodes.setAutoDelete(true);
+	m_markers.setAutoDelete(true);
+
+	// Clear/reset location-markers...
+	m_markers.clear();
+	m_markerCursor.reset();
 
 	// Clear/reset tempo-map...
 	m_nodes.clear();
@@ -69,9 +74,18 @@ void qtractorTimeScale::sync ( const qtractorTimeScale& ts )
 	m_iTicksPerBeat  = ts.m_iTicksPerBeat;
 	m_iPixelsPerBeat = ts.m_iPixelsPerBeat;
 
+	// Copy location markers...
+	m_markers.clear();
+	Marker *pMarker = ts.m_markers.first();
+	while (pMarker) {
+		m_markers.append(new Marker(*pMarker));
+		pMarker = pMarker->next();
+	}
+	m_markerCursor.reset();
+
 	// Copy tempo-map nodes...
 	m_nodes.clear();
-	Node *pNode = ts.m_nodes.first();
+	Node *pNode = ts.nodes().first();
 	while (pNode) {
 		m_nodes.append(new Node(this, pNode->frame,
 			pNode->tempo, pNode->beatType,
@@ -90,6 +104,7 @@ qtractorTimeScale& qtractorTimeScale::copy ( const qtractorTimeScale& ts )
 	if (&ts != this) {
 
 		m_nodes.setAutoDelete(true);
+		m_markers.setAutoDelete(true);
 
 		m_iSampleRate     = ts.m_iSampleRate;
 		m_iSnapPerBeat    = ts.m_iSnapPerBeat;
@@ -385,26 +400,31 @@ void qtractorTimeScale::updateNode ( qtractorTimeScale::Node *pNode )
 	m_cursor.reset(pNode);
 
 	// Update positioning on all nodes thereafter...
-	Node *pPrev = pNode->prev();
-	while (pNode) {
-		if (pPrev) pNode->reset(pPrev);
-		pPrev = pNode;
-		pNode = pNode->next();
+	Node *pNext = pNode;
+	Node *pPrev = pNext->prev();
+	while (pNext) {
+		if (pPrev) pNext->reset(pPrev);
+		pPrev = pNext;
+		pNext = pNext->next();
 	}
+
+	// And update marker/bar positions too...
+	updateMarkers(pNode->prev());
 }
 
 
 void qtractorTimeScale::removeNode ( qtractorTimeScale::Node *pNode )
 {
 	// Don't ever remove the very first node... 
-	Node *pPrev = pNode->prev();
-	if (pPrev == 0)
+	Node *pNodePrev = pNode->prev();
+	if (pNodePrev == 0)
 		return;
 
 	// Relocate internal cursor...
-	m_cursor.reset(pPrev);
+	m_cursor.reset(pNodePrev);
 
 	// Update positioning on all nodes thereafter...
+	Node *pPrev = pNodePrev;
 	Node *pNext = pNode->next();
 	while (pNext) {
 		if (pPrev) pNext->reset(pPrev);
@@ -414,6 +434,9 @@ void qtractorTimeScale::removeNode ( qtractorTimeScale::Node *pNode )
 
 	// Actually remove/unlink the node...
 	m_nodes.remove(pNode);
+
+	// Then update marker/bar positions too...
+	updateMarkers(pNodePrev);
 }
 
 
@@ -426,15 +449,17 @@ void qtractorTimeScale::updateScale (void)
 	m_fFrameRate = 60.0f * float(m_iSampleRate);
 
 	// Update all nodes thereafter...
-	Node *pNode = m_nodes.first();
-	while (pNode) {
-		pNode->update();
-		Node *pPrev = pNode->prev();
-		if (pPrev) pNode->reset(pPrev);
-		//	pNode->pixel = pPrev->pixelFromFrame(pNode->frame);
-		pPrev = pNode;
-		pNode = pNode->next();
+	Node *pPrev = 0;
+	Node *pNext = m_nodes.first();
+	while (pNext) {
+		pNext->update();
+		if (pPrev) pNext->reset(pPrev);
+		pPrev = pNext;
+		pNext = pNext->next();
 	}
+
+	// Also update all marker/bar positions too...
+	updateMarkers(m_nodes.first());
 }
 
 
@@ -599,7 +624,7 @@ QString qtractorTimeScale::textFromTick (
 
 // Beat divisor (snap index) map.
 static int s_aiSnapPerBeat[] = {
-	0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 16, 24, 32, 48, 64, 96, 128
+	0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, 16, 21, 24, 28, 32, 48, 64, 96
 };
 
 const int c_iSnapItemCount = sizeof(s_aiSnapPerBeat) / sizeof(int);
@@ -670,4 +695,143 @@ unsigned long qtractorTimeScale::tickFromFrameRange (
 }
 
 
+// Location marker reset method.
+void qtractorTimeScale::MarkerCursor::reset (
+	qtractorTimeScale::Marker *pMarker )
+{
+	marker = (pMarker ? pMarker : ts->markers().first());
+}
+
+
+// Location marker seek methods.
+qtractorTimeScale::Marker *qtractorTimeScale::MarkerCursor::seekFrame (
+	unsigned long iFrame )
+{
+	if (marker == 0) {
+		marker = ts->markers().first();
+		if (marker == 0)
+			return 0;
+	}
+
+	if (iFrame > marker->frame) {
+		// Seek frame forward...
+		while (marker && marker->next() && iFrame >= (marker->next())->frame)
+			marker = marker->next();
+	}
+	else
+	if (iFrame < marker->frame) {
+		// Seek frame backward...
+		while (marker && marker->frame > iFrame)
+			marker = marker->prev();
+		if (marker == 0)
+			marker = ts->markers().first();
+	}
+
+	return marker;
+}
+
+qtractorTimeScale::Marker *qtractorTimeScale::MarkerCursor::seekBar (
+	unsigned short iBar )
+{
+	return seekFrame(ts->frameFromBar(iBar));
+}
+
+qtractorTimeScale::Marker *qtractorTimeScale::MarkerCursor::seekBeat (
+	unsigned int iBeat )
+{
+	return seekFrame(ts->frameFromBeat(iBeat));
+}
+
+qtractorTimeScale::Marker *qtractorTimeScale::MarkerCursor::seekTick (
+	unsigned long iTick )
+{
+	return seekFrame(ts->frameFromTick(iTick));
+}
+
+qtractorTimeScale::Marker *qtractorTimeScale::MarkerCursor::seekPixel ( int x )
+{
+	return seekFrame(ts->frameFromPixel(x));
+}
+
+
+
+// Location markers list specifics.
+qtractorTimeScale::Marker *qtractorTimeScale::addMarker (
+	unsigned long iFrame, const QString& sText, const QColor& rgbColor )
+{
+	Marker *pMarker	= 0;
+
+	// Snap to nearest bar...
+	unsigned short iBar = 0;
+
+	Node *pNodePrev = m_cursor.seekFrame(iFrame);
+	if (pNodePrev) {
+		iBar = pNodePrev->barFromFrame(iFrame);
+		iFrame = pNodePrev->frameFromBar(iBar);
+	}
+
+	// Seek for the nearest marker...
+	Marker *pMarkerNear = m_markerCursor.seekFrame(iFrame);
+	// Either update existing marker or add new one...
+	if (pMarkerNear && pMarkerNear->frame == iFrame) {
+		// Update exact matching marker...
+		pMarker = pMarkerNear;
+		pMarker->bar = iBar;
+		pMarker->text = sText;
+		pMarker->color = rgbColor;
+	} else {
+		// Add/insert a new marker...
+		pMarker = new Marker(iFrame, iBar, sText, rgbColor);
+		if (pMarkerNear && pMarkerNear->frame > iFrame)
+			m_markers.insertBefore(pMarker, pMarkerNear);
+		else
+		if (pMarkerNear && pMarkerNear->frame < iFrame)
+			m_markers.insertAfter(pMarker, pMarkerNear);
+		else
+			m_markers.append(pMarker);
+	}
+
+	// Update positioning...
+	updateMarker(pMarker);
+
+	return pMarker;
+}
+
+
+void qtractorTimeScale::updateMarker ( qtractorTimeScale::Marker *pMarker )
+{
+	// Relocate internal cursor...
+	m_markerCursor.reset(pMarker);
+}
+
+
+void qtractorTimeScale::removeMarker ( qtractorTimeScale::Marker *pMarker )
+{
+	// Actually remove/unlink the marker
+	// and relocate internal cursor...
+	Marker *pMarkerPrev = pMarker->prev();
+	m_markers.remove(pMarker);
+	m_markerCursor.reset(pMarkerPrev);
+}
+
+
+// Update markers from given node position.
+void qtractorTimeScale::updateMarkers ( qtractorTimeScale::Node *pNode )
+{
+	if (pNode == 0)
+		pNode = m_nodes.first();
+	if (pNode == 0)
+		return;
+
+	Marker *pMarker = m_markerCursor.seekFrame(pNode->frame);
+	while (pMarker) {
+		while (pNode->next() && pMarker->frame > pNode->next()->frame)
+			pNode = pNode->next();
+		if (pMarker->frame >= pNode->frame)
+			pMarker->frame  = pNode->frameFromBar(pMarker->bar);
+		pMarker = pMarker->next();
+	}
+}
+
+
 // end of qtractorTimeScale.cpp
