@@ -32,15 +32,15 @@
 
 
 //----------------------------------------------------------------------
-// class qtractorTimeScaleCommand - implementation.
+// class qtractorTimeScaleNodeCommand - implementation.
 //
 
 // Constructor.
-qtractorTimeScaleCommand::qtractorTimeScaleCommand ( const QString& sName, 
-	qtractorTimeScale *pTimeScale, qtractorTimeScale::Node *pNode,
+qtractorTimeScaleNodeCommand::qtractorTimeScaleNodeCommand (
+	const QString& sName, qtractorTimeScale *pTimeScale,
 	unsigned long iFrame, float fTempo, unsigned short iBeatType,
 	unsigned short iBeatsPerBar, unsigned short iBeatDivisor)
-	: qtractorCommand(sName), m_pTimeScale(pTimeScale), m_pNode(pNode),
+	: qtractorCommand(sName), m_pTimeScale(pTimeScale),
 		m_iFrame(iFrame), m_fTempo(fTempo), m_iBeatType(iBeatType),
 		m_iBeatsPerBar(iBeatsPerBar), m_iBeatDivisor(iBeatDivisor),
 		m_pClipCommand(NULL), m_bAutoTimeStretch(false)
@@ -52,7 +52,7 @@ qtractorTimeScaleCommand::qtractorTimeScaleCommand ( const QString& sName,
 
 
 // Destructor.
-qtractorTimeScaleCommand::~qtractorTimeScaleCommand (void)
+qtractorTimeScaleNodeCommand::~qtractorTimeScaleNodeCommand (void)
 {
 	if (m_pClipCommand)
 		delete m_pClipCommand;
@@ -60,15 +60,12 @@ qtractorTimeScaleCommand::~qtractorTimeScaleCommand (void)
 
 
 // Add time-scale node command method.
-bool qtractorTimeScaleCommand::addNode (void)
+bool qtractorTimeScaleNodeCommand::addNode (void)
 {
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
-	if (m_pNode)
-		return false;
-
 	// If currently playing, we need to do a stop and go...
 	bool bPlaying = pSession->isPlaying();
 	if (bPlaying)
@@ -77,22 +74,18 @@ bool qtractorTimeScaleCommand::addNode (void)
 	if (m_pClipCommand)
 		m_pClipCommand->undo();
 	
-	m_pNode = m_pTimeScale->addNode(
-		m_iFrame,
-		m_fTempo,
-		m_iBeatType,
-		m_iBeatsPerBar,
-		m_iBeatDivisor);
+	qtractorTimeScale::Node *pNode = m_pTimeScale->addNode(
+		m_iFrame, m_fTempo, m_iBeatType, m_iBeatsPerBar, m_iBeatDivisor);
 
 	if (m_pClipCommand) {
 		delete m_pClipCommand;
 		m_pClipCommand = NULL;
 	} else {
-		qtractorTimeScale::Node *pPrev = m_pNode->prev();
+		qtractorTimeScale::Node *pPrev = pNode->prev();
 		float fOldTempo = (pPrev ? pPrev->tempo : m_pTimeScale->tempo());
-		float fNewTempo = m_pNode->tempo;
+		float fNewTempo = pNode->tempo;
 		m_pClipCommand = createClipCommand(name(),
-			m_pNode, fNewTempo, fOldTempo);
+			pNode, fNewTempo, fOldTempo);
 		if (m_pClipCommand)
 			m_pClipCommand->redo();
 	}
@@ -108,21 +101,22 @@ bool qtractorTimeScaleCommand::addNode (void)
 		pSession->unlock();
 	}
 
-	return (m_pNode != NULL);
+	return true;
 }
 
 
 // Update time-scale node command method.
-bool qtractorTimeScaleCommand::updateNode (void)
+bool qtractorTimeScaleNodeCommand::updateNode (void)
 {
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
-	m_pNode = qtractorTimeScale::Cursor(m_pTimeScale).seekFrame(m_iFrame);
-	if (m_pNode == NULL)
+	qtractorTimeScale::Cursor cursor(m_pTimeScale);
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(m_iFrame);
+	if (pNode == NULL)
 		return false;
-	if (m_pNode->frame != m_iFrame)
+	if (pNode->frame != m_iFrame)
 		return false;
 
 	// If currently playing, we need to do a stop and go...
@@ -130,28 +124,28 @@ bool qtractorTimeScaleCommand::updateNode (void)
 	if (bPlaying)
 		pSession->lock();
 
-	float          fTempo       = m_pNode->tempo;
-	unsigned short iBeatType    = m_pNode->beatType;
-	unsigned short iBeatsPerBar = m_pNode->beatsPerBar;
-	unsigned short iBeatDivisor = m_pNode->beatDivisor;
+	float          fTempo       = pNode->tempo;
+	unsigned short iBeatType    = pNode->beatType;
+	unsigned short iBeatsPerBar = pNode->beatsPerBar;
+	unsigned short iBeatDivisor = pNode->beatDivisor;
 
 	if (m_pClipCommand) {
 		m_pClipCommand->undo();
 		delete m_pClipCommand;
 		m_pClipCommand = NULL;
 	} else {
-		float fOldTempo = m_pNode->tempo;
+		float fOldTempo = pNode->tempo;
 		float fNewTempo = m_fTempo;
 		m_pClipCommand = createClipCommand(name(),
-			m_pNode, fNewTempo, fOldTempo);
+			pNode, fNewTempo, fOldTempo);
 	}
 
-	m_pNode->tempo       = m_fTempo;
-	m_pNode->beatType    = m_iBeatType;
-	m_pNode->beatsPerBar = m_iBeatsPerBar;
-	m_pNode->beatDivisor = m_iBeatDivisor;
+	pNode->tempo       = m_fTempo;
+	pNode->beatType    = m_iBeatType;
+	pNode->beatsPerBar = m_iBeatsPerBar;
+	pNode->beatDivisor = m_iBeatDivisor;
 
-	m_pTimeScale->updateNode(m_pNode);
+	m_pTimeScale->updateNode(pNode);
 
 	m_fTempo       = fTempo;
 	m_iBeatType    = iBeatType;
@@ -177,13 +171,17 @@ bool qtractorTimeScaleCommand::updateNode (void)
 
 
 // Remove time-scale node command method.
-bool qtractorTimeScaleCommand::removeNode (void)
+bool qtractorTimeScaleNodeCommand::removeNode (void)
 {
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
-	if (m_pNode == NULL)
+	qtractorTimeScale::Cursor cursor(m_pTimeScale);
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(m_iFrame);
+	if (pNode == NULL)
+		return false;
+	if (pNode->frame != m_iFrame)
 		return false;
 
 	// If currently playing, we need to do a stop and go...
@@ -196,22 +194,20 @@ bool qtractorTimeScaleCommand::removeNode (void)
 		delete m_pClipCommand;
 		m_pClipCommand = NULL;
 	} else {
-		qtractorTimeScale::Node *pPrev = m_pNode->prev();
-		float fOldTempo = m_pNode->tempo;
+		qtractorTimeScale::Node *pPrev = pNode->prev();
+		float fOldTempo = pNode->tempo;
 		float fNewTempo = (pPrev ? pPrev->tempo : m_pTimeScale->tempo());
 		m_pClipCommand = createClipCommand(name(),
-			m_pNode, fNewTempo, fOldTempo);
+			pNode, fNewTempo, fOldTempo);
 	}
 
-	m_iFrame       = m_pNode->frame;
-	m_fTempo       = m_pNode->tempo;
-	m_iBeatType    = m_pNode->beatType;
-	m_iBeatsPerBar = m_pNode->beatsPerBar;
-	m_iBeatDivisor = m_pNode->beatDivisor;
+//	m_iFrame       = pNode->frame;
+	m_fTempo       = pNode->tempo;
+	m_iBeatType    = pNode->beatType;
+	m_iBeatsPerBar = pNode->beatsPerBar;
+	m_iBeatDivisor = pNode->beatDivisor;
 
-	m_pTimeScale->removeNode(m_pNode);
-
-	m_pNode = NULL;
+	m_pTimeScale->removeNode(pNode);
 
 	if (m_pClipCommand)
 		m_pClipCommand->redo();
@@ -232,7 +228,7 @@ bool qtractorTimeScaleCommand::removeNode (void)
 
 
 // Make it automatic clip time-stretching command (static).
-qtractorClipCommand *qtractorTimeScaleCommand::createClipCommand (
+qtractorClipCommand *qtractorTimeScaleNodeCommand::createClipCommand (
 	const QString& sName, qtractorTimeScale::Node *pNode,
 	float fNewTempo, float fOldTempo )
 {
@@ -295,12 +291,13 @@ qtractorTimeScaleAddNodeCommand::qtractorTimeScaleAddNodeCommand (
 	qtractorTimeScale *pTimeScale, unsigned long iFrame,
 	float fTempo, unsigned short iBeatType,
 	unsigned short iBeatsPerBar, unsigned short iBeatDivisor )
-	: qtractorTimeScaleCommand(QObject::tr("add tempo node"), pTimeScale,
-		NULL, iFrame, fTempo, iBeatType, iBeatsPerBar, iBeatDivisor)
+	: qtractorTimeScaleNodeCommand(
+		QObject::tr("add tempo node"), pTimeScale,
+		iFrame, fTempo, iBeatType, iBeatsPerBar, iBeatDivisor)
 {
 }
 
-// Time-scale command methods.
+// Time-scale node command methods.
 bool qtractorTimeScaleAddNodeCommand::redo (void) { return addNode(); }
 bool qtractorTimeScaleAddNodeCommand::undo (void) { return removeNode(); }
 
@@ -314,12 +311,13 @@ qtractorTimeScaleUpdateNodeCommand::qtractorTimeScaleUpdateNodeCommand (
 	qtractorTimeScale *pTimeScale, unsigned long iFrame,
 	float fTempo, unsigned short iBeatType,
 	unsigned short iBeatsPerBar, unsigned short iBeatDivisor)
-	: qtractorTimeScaleCommand(QObject::tr("update tempo node"), pTimeScale,
-		NULL, iFrame, fTempo, iBeatType, iBeatsPerBar, iBeatDivisor)
+	: qtractorTimeScaleNodeCommand(
+		QObject::tr("update tempo node"), pTimeScale,
+		iFrame, fTempo, iBeatType, iBeatsPerBar, iBeatDivisor)
 {
 }
 
-// Time-scale command methods.
+// Time-scale node command methods.
 bool qtractorTimeScaleUpdateNodeCommand::redo (void) { return updateNode(); }
 bool qtractorTimeScaleUpdateNodeCommand::undo (void) { return redo(); }
 
@@ -331,13 +329,198 @@ bool qtractorTimeScaleUpdateNodeCommand::undo (void) { return redo(); }
 // Constructor.
 qtractorTimeScaleRemoveNodeCommand::qtractorTimeScaleRemoveNodeCommand (
 	qtractorTimeScale *pTimeScale, qtractorTimeScale::Node *pNode )
-	: qtractorTimeScaleCommand(QObject::tr("remove tempo node"), pTimeScale, pNode)
+	: qtractorTimeScaleNodeCommand(
+		QObject::tr("remove tempo node"), pTimeScale, pNode->frame)
 {
 }
 
-// Time-scale command methods.
+// Time-scale node command methods.
 bool qtractorTimeScaleRemoveNodeCommand::redo (void) { return removeNode(); }
 bool qtractorTimeScaleRemoveNodeCommand::undo (void) { return addNode(); }
 
 
+//----------------------------------------------------------------------
+// class qtractorTimeScaleMarkerCommand - implementation.
+//
+
+// Constructor.
+qtractorTimeScaleMarkerCommand::qtractorTimeScaleMarkerCommand (
+	const QString& sName, qtractorTimeScale *pTimeScale,
+	unsigned long iFrame, const QString& sText, const QColor& rgbColor )
+	: qtractorCommand(sName), m_pTimeScale(pTimeScale),
+		m_iFrame(iFrame), m_sText(sText), m_rgbColor(rgbColor)
+{
+}
+
+
+// Add time-scale marker command method.
+bool qtractorTimeScaleMarkerCommand::addMarker (void)
+{
+	return (m_pTimeScale->addMarker(m_iFrame, m_sText, m_rgbColor) != NULL);
+}
+
+
+// Update time-scale marker command method.
+bool qtractorTimeScaleMarkerCommand::updateMarker (void)
+{
+	qtractorTimeScale::Marker *pMarker
+		= m_pTimeScale->markers().seekFrame(m_iFrame);
+	if (pMarker == NULL)
+		return false;
+	if (pMarker->frame != m_iFrame)
+		return false;
+
+	QString sText    = pMarker->text;
+	QColor  rgbColor = pMarker->color;
+
+	pMarker->text  = m_sText;
+	pMarker->color = m_rgbColor;
+
+	m_pTimeScale->updateMarker(pMarker);
+
+	m_sText    = sText;
+	m_rgbColor = rgbColor;
+
+	return true;
+}
+
+
+// Remove time-scale marker command method.
+bool qtractorTimeScaleMarkerCommand::removeMarker (void)
+{
+	qtractorTimeScale::Marker *pMarker
+		= m_pTimeScale->markers().seekFrame(m_iFrame);
+	if (pMarker == NULL)
+		return false;
+	if (pMarker->frame != m_iFrame)
+		return false;
+
+//	m_iFrame   = pMarker->frame;
+	m_sText    = pMarker->text;
+	m_rgbColor = pMarker->color;
+
+	m_pTimeScale->removeMarker(pMarker);
+
+	return true;
+}
+
+
+//----------------------------------------------------------------------
+// class qtractorTimeScaleAddMarkerCommand - implementation.
+//
+
+// Constructor.
+qtractorTimeScaleAddMarkerCommand::qtractorTimeScaleAddMarkerCommand (
+	qtractorTimeScale *pTimeScale, unsigned long iFrame,
+	const QString& sText, const QColor& rgbColor )
+	: qtractorTimeScaleMarkerCommand(
+		QObject::tr("add marker"), pTimeScale,
+			iFrame, sText, rgbColor)
+{
+}
+
+// Time-scale marker command methods.
+bool qtractorTimeScaleAddMarkerCommand::redo (void) { return addMarker(); }
+bool qtractorTimeScaleAddMarkerCommand::undo (void) { return removeMarker(); }
+
+
+//----------------------------------------------------------------------
+// class qtractorTimeScaleUpdateMarkerCommand - implementation.
+//
+
+// Constructor.
+qtractorTimeScaleUpdateMarkerCommand::qtractorTimeScaleUpdateMarkerCommand (
+	qtractorTimeScale *pTimeScale, unsigned long iFrame,
+	const QString& sText, const QColor& rgbColor )
+	: qtractorTimeScaleMarkerCommand(
+		QObject::tr("update marker"), pTimeScale,
+			iFrame, sText, rgbColor)
+{
+}
+
+// Time-scale marker command methods.
+bool qtractorTimeScaleUpdateMarkerCommand::redo (void) { return updateMarker(); }
+bool qtractorTimeScaleUpdateMarkerCommand::undo (void) { return redo(); }
+
+
+//----------------------------------------------------------------------
+// class qtractorTimeScaleRemoveMarkerCommand - implementation.
+//
+
+// Constructor.
+qtractorTimeScaleRemoveMarkerCommand::qtractorTimeScaleRemoveMarkerCommand (
+	qtractorTimeScale *pTimeScale, qtractorTimeScale::Marker *pMarker )
+	: qtractorTimeScaleMarkerCommand(
+		QObject::tr("remove marker"), pTimeScale, pMarker->frame)
+{
+}
+
+// Time-scale marker command methods.
+bool qtractorTimeScaleRemoveMarkerCommand::redo (void) { return removeMarker(); }
+bool qtractorTimeScaleRemoveMarkerCommand::undo (void) { return addMarker(); }
+
+
+
+//----------------------------------------------------------------------
+// class qtractorTimeScaleMoveMarkerCommand - implementation.
+//
+
+// Constructor.
+qtractorTimeScaleMoveMarkerCommand::qtractorTimeScaleMoveMarkerCommand (
+	qtractorTimeScale *pTimeScale, qtractorTimeScale::Marker *pMarker,
+	unsigned long iFrame ) : qtractorTimeScaleMarkerCommand(
+		QObject::tr("move marker"), pTimeScale,
+		pMarker->frame, pMarker->text, pMarker->color)
+{
+	// The new location.
+	iFrame = pTimeScale->frameFromBar(pTimeScale->barFromFrame(iFrame));
+
+	m_iNewFrame = iFrame;
+	m_iOldFrame = pMarker->frame;
+
+	// Replaced marker salvage.
+	pMarker	= pTimeScale->markers().seekFrame(iFrame);
+	if (pMarker && pMarker->frame == iFrame) {
+		m_bOldMarker  = true;
+		m_sOldText    = pMarker->text;
+		m_rgbOldColor = pMarker->color;
+	} else {
+		m_bOldMarker  = false;
+	}
+}
+
+// Time-scale marker command methods.
+bool qtractorTimeScaleMoveMarkerCommand::redo (void)
+{
+	qtractorTimeScale *pTimeScale = timeScale();
+	if (pTimeScale == NULL)
+		return false;
+
+	unsigned long iNewFrame = m_iNewFrame;
+	unsigned long iOldFrame = m_iOldFrame;
+
+	qtractorTimeScale::Marker *pMarker
+		= pTimeScale->markers().seekFrame(iOldFrame);
+	if (pMarker && pMarker->frame == iOldFrame)
+		pTimeScale->removeMarker(pMarker);
+
+	pTimeScale->addMarker(iNewFrame, text(), color());
+
+	m_iNewFrame = iOldFrame;
+	m_iOldFrame = iNewFrame;
+
+	return true;
+}
+
+bool qtractorTimeScaleMoveMarkerCommand::undo (void)
+{
+	bool bResult = redo();
+
+	if (bResult && m_bOldMarker)
+		timeScale()->addMarker(m_iNewFrame, m_sOldText, m_rgbOldColor);
+
+	return bResult;
+}
+
+
 // end of qtractorTimeScaleCommand.cpp
