@@ -1,7 +1,7 @@
 // qtractorTrack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2012, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -1035,7 +1035,13 @@ unsigned long qtractorTrack::clipRecordEnd ( unsigned long iFrameTime ) const
 	unsigned long iClipRecordEnd = 0;
 
 	if (m_pSession && m_pSession->isPunching()) {
-		iClipRecordEnd += m_pSession->punchOut();
+		unsigned long iPunchOut  = m_pSession->punchOut();
+		unsigned long iLoopStart = m_pSession->loopStart();
+		unsigned long iLoopEnd   = m_pSession->loopEnd();
+		if (iLoopStart < iLoopEnd && iPunchOut >= iLoopEnd)
+			iClipRecordEnd += iLoopEnd;
+		else
+			iClipRecordEnd += iPunchOut;
 	} else {
 		iClipRecordEnd += iFrameTime;
 		if (iClipRecordEnd  > m_iClipRecordStart)
@@ -2104,4 +2110,4 @@ void qtractorTrack::applyCurveFile ( qtractorCurveFile *pCurveFile ) const
 }
 
 
-// end of qtractorTrack.h
+// end of qtractorTrack.cpp
