@@ -1,7 +1,7 @@
 // qtractorTrackTime.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2012, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -24,6 +24,8 @@
 
 #include "qtractorScrollView.h"
 
+#include "qtractorTimeScale.h"
+
 #include <QPixmap>
 
 
@@ -46,8 +48,6 @@ public:
 
 	// Constructor.
 	qtractorTrackTime(qtractorTracks *pTracks, QWidget *pParent = 0);
-	// Destructor.
-	~qtractorTrackTime();
 
 	// Rectangular contents update.
 	void updateContents(const QRect& rect);
@@ -82,6 +82,13 @@ protected:
 	// Context menu request slot (dummy).
 	void contextMenuEvent(QContextMenuEvent *);
 
+	// Trap for help/tool-tip events.
+	bool eventFilter(QObject *pObject, QEvent *pEvent);
+
+	// Show dragging tooltip...
+	void showToolTip(unsigned long iFrame) const;
+	void showToolTip(const QRect& rect) const;
+
 protected slots:
 
 	// To have timeline in h-sync with main track view.
@@ -101,13 +108,16 @@ private:
 	// The current selecting/dragging head stuff.
 	enum DragState {
 		DragNone = 0, DragStart, DragSelect,
-		DragPlayHead, DragEditHead, DragEditTail,
+		DragPlayHead, DragMarker,
+		DragEditHead, DragEditTail,
 		DragLoopStart, DragLoopEnd,
 		DragPunchIn, DragPunchOut
 	} m_dragState, m_dragCursor;
 
 	QRect  m_rectDrag;
 	QPoint m_posDrag;
+
+	qtractorTimeScale::Marker *m_pDragMarker;
 };
 
 
