@@ -114,6 +114,9 @@ public:
 	void selectAll(bool bSelect = true);
 	void selectInvert();
 
+	// Insertion method.
+	bool insertEditRange(qtractorTrack *pTrack = NULL);
+
 	// Simple main-form redirectors.
 	void selectionChangeNotify();
 	void contentsChangeNotify();
@@ -172,6 +175,11 @@ protected:
 
 	bool rangeClipEx(qtractorClip *pClip, bool bLoopSet);
 
+	// Insertion method (track).
+	int insertEditRangeTrack(
+		qtractorClipCommand *pClipCommand, qtractorTrack *pTrack,
+		unsigned long iInsertStart, unsigned long iInsertEnd) const;
+
 public slots:
 
 	// Track-view update (obviously a slot).
